@@ -1,6 +1,10 @@
 #ifndef MC_GLIBCOMPAT_H
 #define MC_GLIBCOMPAT_H
 
+#if ! GLIB_CHECK_VERSION (2, 22, 0)
+#include <stdarg.h>             /* for g_error_new_valist() */
+#endif /* ! GLIB_CHECK_VERSION (2, 22, 0) */
+
 /*** typedefs(not structures) and defined constants **********************************************/
 
 /*** enums ***************************************************************************************/
@@ -11,14 +15,14 @@
 
 /*** declarations of public functions ************************************************************/
 
-#if ! GLIB_CHECK_VERSION (2, 13, 0)
-gboolean g_unichar_iszerowidth (gunichar);
-#endif /* ! GLIB_CHECK_VERSION (2, 13, 0) */
-
 #if ! GLIB_CHECK_VERSION (2, 16, 0)
 int g_strcmp0 (const char *str1, const char *str2);
 #endif /* ! GLIB_CHECK_VERSION (2, 16, 0) */
 
+#if ! GLIB_CHECK_VERSION (2, 22, 0)
+GError *g_error_new_valist (GQuark domain, gint code, const gchar * format, va_list args);
+#endif /* ! GLIB_CHECK_VERSION (2, 22, 0) */
+
 #if ! GLIB_CHECK_VERSION (2, 28, 0)
 void g_slist_free_full (GSList * list, GDestroyNotify free_func);
 void g_list_free_full (GList * list, GDestroyNotify free_func);
