@@ -2,13 +2,13 @@
    Search text engine.
    Regex search
 
-   Copyright (C) 2009-2014
+   Copyright (C) 2009-2015
    Free Software Foundation, Inc.
 
    Written by:
    Slava Zanko <slavazanko@gmail.com>, 2009, 2010, 2011, 2013
    Vitaliy Filippov <vitalif@yourcmc.ru>, 2011
-   Andrew Borodin <aborodin@vmail.ru>, 2013
+   Andrew Borodin <aborodin@vmail.ru>, 2013-2015
 
    This file is part of the Midnight Commander.
 
@@ -61,13 +61,11 @@ typedef enum
 
 /*** file scope functions ************************************************************************/
 
+#ifndef SEARCH_TYPE_GLIB
 static gboolean
 mc_search__regex_str_append_if_special (GString * copy_to, const GString * regex_str,
                                         gsize * offset)
 {
-    char *tmp_regex_str;
-    gsize spec_chr_len;
-    const char **spec_chr;
     const char *special_chars[] = {
         "\\s", "\\S",
         "\\d", "\\D",
@@ -83,39 +81,41 @@ mc_search__regex_str_append_if_special (GString * copy_to, const GString * regex
         "\\E", "\\Q",
         NULL
     };
-    spec_chr = special_chars;
+
+    char *tmp_regex_str;
+    const char **spec_chr;
 
     tmp_regex_str = &(regex_str->str[*offset]);
 
-    while (*spec_chr)
+    for (spec_chr = special_chars; *spec_chr != NULL; spec_chr++)
     {
-        spec_chr_len = strlen (*spec_chr);
-        if (!strncmp (tmp_regex_str, *spec_chr, spec_chr_len))
-        {
-            if (!strutils_is_char_escaped (regex_str->str, tmp_regex_str))
-            {
-                if (!strncmp ("\\x", *spec_chr, spec_chr_len))
-                {
-                    if (*(tmp_regex_str + spec_chr_len) == '{')
-                    {
-                        while ((spec_chr_len < regex_str->len - *offset)
-                               && *(tmp_regex_str + spec_chr_len) != '}')
-                            spec_chr_len++;
-                        if (*(tmp_regex_str + spec_chr_len) == '}')
-                            spec_chr_len++;
-                    }
-                    else
-                        spec_chr_len += 2;
-                }
-                g_string_append_len (copy_to, tmp_regex_str, spec_chr_len);
-                *offset += spec_chr_len;
-                return TRUE;
-            }
-        }
-        spec_chr++;
-    }
-    return FALSE;
+        gsize spec_chr_len;
 
+        spec_chr_len = strlen (*spec_chr);
+
+        if (strncmp (tmp_regex_str, *spec_chr, spec_chr_len) == 0
+            && !strutils_is_char_escaped (regex_str->str, tmp_regex_str))
+        {
+            if (strncmp ("\\x", *spec_chr, spec_chr_len) == 0)
+            {
+                if (tmp_regex_str[spec_chr_len] != '{')
+                    spec_chr_len += 2;
+                else
+                {
+                    while ((spec_chr_len < regex_str->len - *offset)
+                           && tmp_regex_str[spec_chr_len] != '}')
+                        spec_chr_len++;
+                    if (tmp_regex_str[spec_chr_len] == '}')
+                        spec_chr_len++;
+                }
+            }
+            g_string_append_len (copy_to, tmp_regex_str, spec_chr_len);
+            *offset += spec_chr_len;
+            return TRUE;
+        }
+    }
+
+    return FALSE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -245,6 +245,66 @@ mc_search__cond_struct_new_regex_ci_str (const char *charset, const GString * as
 
     return ret_str;
 }
+#endif /* !SEARCH_TYPE_GLIB */
+
+/* --------------------------------------------------------------------------------------------- */
+
+#ifdef SEARCH_TYPE_GLIB
+/* A thin wrapper above g_regex_match_full that makes sure the string passed
+ * to it is valid UTF-8 (unless G_REGEX_RAW compile flag was set), as it is a
+ * requirement by glib and it might crash otherwise. See: mc ticket 3449.
+ * Be careful: there might be embedded NULs in the strings. */
+static gboolean
+mc_search__g_regex_match_full_safe (const GRegex * regex,
+                                    const gchar * string,
+                                    gssize string_len,
+                                    gint start_position,
+                                    GRegexMatchFlags match_options,
+                                    GMatchInfo ** match_info, GError ** error)
+{
+    char *string_safe, *p, *end;
+    gboolean ret;
+
+    if ((g_regex_get_compile_flags (regex) & G_REGEX_RAW)
+        || g_utf8_validate (string, string_len, NULL))
+    {
+        return g_regex_match_full (regex, string, string_len, start_position, match_options,
+                                   match_info, error);
+    }
+
+    if (string_len < 0)
+    {
+        string_len = strlen (string);
+    }
+
+    /* Correctly handle embedded NULs while copying */
+    p = string_safe = g_malloc (string_len);
+    memcpy (string_safe, string, string_len);
+    end = p + string_len;
+
+    while (p < end)
+    {
+        gunichar c = g_utf8_get_char_validated (p, -1);
+        if (c != (gunichar) (-1) && c != (gunichar) (-2))
+        {
+            p = g_utf8_next_char (p);
+        }
+        else
+        {
+            /* U+FFFD would be the proper choice, but then we'd have to
+               maintain mapping between old and new offsets.
+               So rather do a byte by byte replacement. */
+            *p++ = '\0';
+        }
+    }
+
+    ret =
+        g_regex_match_full (regex, string_safe, string_len, start_position, match_options,
+                            match_info, error);
+    g_free (string_safe);
+    return ret;
+}
+#endif /* SEARCH_TYPE_GLIB */
 
 /* --------------------------------------------------------------------------------------------- */
 
@@ -255,8 +315,9 @@ mc_search__regex_found_cond_one (mc_search_t * lc_mc_search, mc_search_regex_t *
 #ifdef SEARCH_TYPE_GLIB
     GError *mcerror = NULL;
 
-    if (!g_regex_match_full (regex, search_str->str, search_str->len, 0, G_REGEX_MATCH_NEWLINE_ANY,
-                             &lc_mc_search->regex_match_info, &mcerror))
+    if (!mc_search__g_regex_match_full_safe
+        (regex, search_str->str, search_str->len, 0, G_REGEX_MATCH_NEWLINE_ANY,
+         &lc_mc_search->regex_match_info, &mcerror))
     {
         g_match_info_free (lc_mc_search->regex_match_info);
         lc_mc_search->regex_match_info = NULL;
@@ -368,8 +429,8 @@ mc_search_regex__get_token_by_num (const mc_search_t * lc_mc_search, gsize lc_in
     fnd_end = lc_mc_search->iovector[lc_index * 2 + 1];
 #endif /* SEARCH_TYPE_GLIB */
 
-    if (fnd_end - fnd_start == 0)
-        return NULL;
+    if (fnd_end == fnd_start)
+        return g_strdup ("");
 
     return g_strndup (lc_mc_search->regex_buffer->str + fnd_start, fnd_end - fnd_start);
 
@@ -382,17 +443,18 @@ mc_search_regex__replace_handle_esc_seq (const GString * replace_str, const gsiz
                                          gsize * skip_len, int *ret)
 {
     char *curr_str = &(replace_str->str[current_pos]);
-    char c = *(curr_str + 1);
+    char c = curr_str[1];
 
     if (replace_str->len > current_pos + 2)
     {
         if (c == '{')
         {
             for (*skip_len = 2; /* \{ */
-                 current_pos + *skip_len < replace_str->len
-                 && *(curr_str + *skip_len) >= '0'
-                 && *(curr_str + *skip_len) <= '7'; (*skip_len)++);
-            if (current_pos + *skip_len < replace_str->len && *(curr_str + *skip_len) == '}')
+                 current_pos + *skip_len < replace_str->len && curr_str[*skip_len] >= '0'
+                 && curr_str[*skip_len] <= '7'; (*skip_len)++)
+                ;
+
+            if (current_pos + *skip_len < replace_str->len && curr_str[*skip_len] == '}')
             {
                 (*skip_len)++;
                 *ret = REPLACE_PREPARE_T_ESCAPE_SEQ;
@@ -408,13 +470,15 @@ mc_search_regex__replace_handle_esc_seq (const GString * replace_str, const gsiz
         if (c == 'x')
         {
             *skip_len = 2;      /* \x */
-            c = *(curr_str + 2);
+            c = curr_str[2];
             if (c == '{')
             {
                 for (*skip_len = 3;     /* \x{ */
                      current_pos + *skip_len < replace_str->len
-                     && g_ascii_isxdigit ((guchar) * (curr_str + *skip_len)); (*skip_len)++);
-                if (current_pos + *skip_len < replace_str->len && *(curr_str + *skip_len) == '}')
+                     && g_ascii_isxdigit ((guchar) curr_str[*skip_len]); (*skip_len)++)
+                    ;
+
+                if (current_pos + *skip_len < replace_str->len && curr_str[*skip_len] == '}')
                 {
                     (*skip_len)++;
                     *ret = REPLACE_PREPARE_T_ESCAPE_SEQ;
@@ -434,7 +498,7 @@ mc_search_regex__replace_handle_esc_seq (const GString * replace_str, const gsiz
             }
             else
             {
-                c = *(curr_str + 3);
+                c = curr_str[3];
                 if (!g_ascii_isxdigit ((guchar) c))
                     *skip_len = 3;      /* \xH */
                 else
@@ -468,8 +532,8 @@ mc_search_regex__process_replace_str (const GString * replace_str, const gsize c
 
     *skip_len = 0;
 
-    if ((*curr_str == '$') && (*(curr_str + 1) == '{') && ((*(curr_str + 2) & (char) 0xf0) == 0x30)
-        && (replace_str->len > current_pos + 2))
+    if (replace_str->len > current_pos + 2 && curr_str[0] == '$' && curr_str[1] == '{'
+        && (curr_str[2] & (char) 0xf0) == 0x30)
     {
         char *tmp_str;
 
@@ -481,9 +545,10 @@ mc_search_regex__process_replace_str (const GString * replace_str, const gsize c
 
         for (*skip_len = 0;
              current_pos + *skip_len + 2 < replace_str->len
-             && (*(curr_str + 2 + *skip_len) & (char) 0xf0) == 0x30; (*skip_len)++);
+             && (curr_str[2 + *skip_len] & (char) 0xf0) == 0x30; (*skip_len)++)
+            ;
 
-        if (*(curr_str + 2 + *skip_len) != '}')
+        if (curr_str[2 + *skip_len] != '}')
             return REPLACE_PREPARE_T_NOTHING_SPECIAL;
 
         tmp_str = g_strndup (curr_str + 2, *skip_len);
@@ -497,7 +562,7 @@ mc_search_regex__process_replace_str (const GString * replace_str, const gsize c
         return ret;             /* capture buffer index >= 0 */
     }
 
-    if ((*curr_str == '\\') && (replace_str->len > current_pos + 1))
+    if (curr_str[0] == '\\' && replace_str->len > current_pos + 1)
     {
         if (strutils_is_char_escaped (replace_str->str, curr_str))
         {
@@ -505,9 +570,9 @@ mc_search_regex__process_replace_str (const GString * replace_str, const gsize c
             return REPLACE_PREPARE_T_NOTHING_SPECIAL;
         }
 
-        if (g_ascii_isdigit (*(curr_str + 1)))
+        if (g_ascii_isdigit (curr_str[1]))
         {
-            ret = g_ascii_digit_value (*(curr_str + 1));        /* capture buffer index >= 0 */
+            ret = g_ascii_digit_value (curr_str[1]);    /* capture buffer index >= 0 */
             *skip_len = 2;      /* \\ and one digit */
             return ret;
         }
@@ -517,7 +582,8 @@ mc_search_regex__process_replace_str (const GString * replace_str, const gsize c
 
         ret = REPLACE_PREPARE_T_REPLACE_FLAG;
         *skip_len += 2;
-        switch (*(curr_str + 1))
+
+        switch (curr_str[1])
         {
         case 'U':
             *replace_flags |= REPLACE_T_UPP_TRANSFORM;
@@ -556,7 +622,7 @@ mc_search_regex__process_append_str (GString * dest_str, const char *from, gsize
     if (len == (gsize) (-1))
         len = strlen (from);
 
-    if ((*replace_flags == REPLACE_T_NO_TRANSFORM) != 0)
+    if (*replace_flags == REPLACE_T_NO_TRANSFORM)
     {
         g_string_append_len (dest_str, from, len);
         return;
@@ -616,6 +682,7 @@ mc_search_regex__process_escape_sequence (GString * dest_str, const char *from,
         len = strlen (from);
     if (len == 0)
         return;
+
     if (from[i] == '{')
         i++;
     if (i >= len)
@@ -703,6 +770,28 @@ mc_search_regex__process_escape_sequence (GString * dest_str, const char *from,
     }
 }
 
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Get regex flags for compilation of expressions.
+ * @param charset   the charset
+ *
+ * @return regex flags
+ */
+
+static GRegexCompileFlags
+mc_search__regex_get_compile_flags (const char *charset, gboolean is_case_sensitive)
+{
+    GRegexCompileFlags g_regex_options = G_REGEX_OPTIMIZE | G_REGEX_DOTALL;
+
+    if (!(mc_global.utf8_display && str_isutf8 (charset)))
+        g_regex_options |= G_REGEX_RAW;
+
+    if (!is_case_sensitive)
+        g_regex_options |= G_REGEX_CASELESS;
+
+    return g_regex_options;
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
@@ -711,72 +800,79 @@ void
 mc_search__cond_struct_new_init_regex (const char *charset, mc_search_t * lc_mc_search,
                                        mc_search_cond_t * mc_search_cond)
 {
+    if (lc_mc_search->whole_words && !lc_mc_search->is_entire_line)
+    {
+        /* NOTE: \b as word boundary doesn't allow search
+         * whole words with non-ASCII symbols.
+         * Update: Is it still true nowadays? Probably not. #2396, #3524 */
+        g_string_prepend (mc_search_cond->str, "(?<![\\p{L}\\p{N}_])");
+        g_string_append (mc_search_cond->str, "(?![\\p{L}\\p{N}_])");
+    }
+
+    {
 #ifdef SEARCH_TYPE_GLIB
-    GError *mcerror = NULL;
+        GError *mcerror = NULL;
 
-    if (!lc_mc_search->is_case_sensitive)
-    {
-        GString *tmp;
+        mc_search_cond->regex_handle =
+            g_regex_new (mc_search_cond->str->str,
+                         mc_search__regex_get_compile_flags (charset,
+                                                             lc_mc_search->is_case_sensitive), 0,
+                         &mcerror);
 
-        tmp = mc_search_cond->str;
-        mc_search_cond->str = mc_search__cond_struct_new_regex_ci_str (charset, tmp);
-        g_string_free (tmp, TRUE);
-    }
-    mc_search_cond->regex_handle =
-        g_regex_new (mc_search_cond->str->str, G_REGEX_OPTIMIZE | G_REGEX_RAW | G_REGEX_DOTALL,
-                     0, &mcerror);
-
-    if (mcerror != NULL)
-    {
-        lc_mc_search->error = MC_SEARCH_E_REGEX_COMPILE;
-        lc_mc_search->error_str = str_conv_gerror_message (mcerror, _("Regular expression error"));
-        g_error_free (mcerror);
-        return;
-    }
-#else /* SEARCH_TYPE_GLIB */
-    const char *error;
-    int erroffset;
-    int pcre_options = PCRE_EXTRA | PCRE_MULTILINE;
-
-    if (str_isutf8 (charset) && mc_global.utf8_display)
-    {
-        pcre_options |= PCRE_UTF8;
-        if (!lc_mc_search->is_case_sensitive)
-            pcre_options |= PCRE_CASELESS;
-    }
-    else
-    {
-        if (!lc_mc_search->is_case_sensitive)
+        if (mcerror != NULL)
         {
-            GString *tmp;
-
-            tmp = mc_search_cond->str;
-            mc_search_cond->str = mc_search__cond_struct_new_regex_ci_str (charset, tmp);
-            g_string_free (tmp, TRUE);
+            lc_mc_search->error = MC_SEARCH_E_REGEX_COMPILE;
+            lc_mc_search->error_str =
+                str_conv_gerror_message (mcerror, _("Regular expression error"));
+            g_error_free (mcerror);
+            return;
         }
-    }
+#else /* SEARCH_TYPE_GLIB */
+        const char *error;
+        int erroffset;
+        int pcre_options = PCRE_EXTRA | PCRE_MULTILINE;
 
-    mc_search_cond->regex_handle =
-        pcre_compile (mc_search_cond->str->str, pcre_options, &error, &erroffset, NULL);
-    if (mc_search_cond->regex_handle == NULL)
-    {
-        lc_mc_search->error = MC_SEARCH_E_REGEX_COMPILE;
-        lc_mc_search->error_str = g_strdup (error);
-        return;
-    }
-    lc_mc_search->regex_match_info = pcre_study (mc_search_cond->regex_handle, 0, &error);
-    if (lc_mc_search->regex_match_info == NULL)
-    {
-        if (error)
+        if (str_isutf8 (charset) && mc_global.utf8_display)
+        {
+            pcre_options |= PCRE_UTF8;
+            if (!lc_mc_search->is_case_sensitive)
+                pcre_options |= PCRE_CASELESS;
+        }
+        else
+        {
+            if (!lc_mc_search->is_case_sensitive)
+            {
+                GString *tmp;
+
+                tmp = mc_search_cond->str;
+                mc_search_cond->str = mc_search__cond_struct_new_regex_ci_str (charset, tmp);
+                g_string_free (tmp, TRUE);
+            }
+        }
+
+        mc_search_cond->regex_handle =
+            pcre_compile (mc_search_cond->str->str, pcre_options, &error, &erroffset, NULL);
+        if (mc_search_cond->regex_handle == NULL)
         {
             lc_mc_search->error = MC_SEARCH_E_REGEX_COMPILE;
             lc_mc_search->error_str = g_strdup (error);
-            g_free (mc_search_cond->regex_handle);
-            mc_search_cond->regex_handle = NULL;
             return;
         }
-    }
+        lc_mc_search->regex_match_info = pcre_study (mc_search_cond->regex_handle, 0, &error);
+        if (lc_mc_search->regex_match_info == NULL)
+        {
+            if (error != NULL)
+            {
+                lc_mc_search->error = MC_SEARCH_E_REGEX_COMPILE;
+                lc_mc_search->error_str = g_strdup (error);
+                g_free (mc_search_cond->regex_handle);
+                mc_search_cond->regex_handle = NULL;
+                return;
+            }
+        }
 #endif /* SEARCH_TYPE_GLIB */
+    }
+
     lc_mc_search->is_utf8 = str_isutf8 (charset);
 }
 
@@ -859,21 +955,10 @@ mc_search__run_regex (mc_search_t * lc_mc_search, const void *user_data,
         {
         case COND__FOUND_OK:
 #ifdef SEARCH_TYPE_GLIB
-            if (lc_mc_search->whole_words)
-                g_match_info_fetch_pos (lc_mc_search->regex_match_info, 2, &start_pos, &end_pos);
-            else
-                g_match_info_fetch_pos (lc_mc_search->regex_match_info, 0, &start_pos, &end_pos);
+            g_match_info_fetch_pos (lc_mc_search->regex_match_info, 0, &start_pos, &end_pos);
 #else /* SEARCH_TYPE_GLIB */
-            if (lc_mc_search->whole_words)
-            {
-                start_pos = lc_mc_search->iovector[4];
-                end_pos = lc_mc_search->iovector[5];
-            }
-            else
-            {
-                start_pos = lc_mc_search->iovector[0];
-                end_pos = lc_mc_search->iovector[1];
-            }
+            start_pos = lc_mc_search->iovector[0];
+            end_pos = lc_mc_search->iovector[1];
 #endif /* SEARCH_TYPE_GLIB */
             if (found_len != NULL)
                 *found_len = end_pos - start_pos;
@@ -913,12 +998,10 @@ GString *
 mc_search_regex_prepare_replace_str (mc_search_t * lc_mc_search, GString * replace_str)
 {
     GString *ret;
-    gchar *tmp_str;
 
     int num_replace_tokens;
     gsize loop;
-    gsize len = 0;
-    gchar *prev_str;
+    gsize prev = 0;
     replace_transform_type_t replace_flags = REPLACE_T_NO_TRANSFORM;
 
     num_replace_tokens =
@@ -936,11 +1019,12 @@ mc_search_regex_prepare_replace_str (mc_search_t * lc_mc_search, GString * repla
     }
 
     ret = g_string_sized_new (64);
-    prev_str = replace_str->str;
 
     for (loop = 0; loop < replace_str->len - 1; loop++)
     {
         int lc_index;
+        gchar *tmp_str;
+        gsize len = 0;
 
         lc_index = mc_search_regex__process_replace_str (replace_str, loop, &len, &replace_flags);
 
@@ -948,39 +1032,37 @@ mc_search_regex_prepare_replace_str (mc_search_t * lc_mc_search, GString * repla
         {
             if (len != 0)
             {
-                mc_search_regex__process_append_str (ret, prev_str,
-                                                     replace_str->str - prev_str + loop,
+                mc_search_regex__process_append_str (ret, replace_str->str + prev, loop - prev,
                                                      &replace_flags);
                 mc_search_regex__process_append_str (ret, replace_str->str + loop + 1, len - 1,
                                                      &replace_flags);
-                prev_str = replace_str->str + loop + len;
-                loop += len - 1;
+                prev = loop + len;
+                loop = prev - 1;        /* prepare to loop++ */
             }
+
             continue;
         }
 
         if (lc_index == REPLACE_PREPARE_T_REPLACE_FLAG)
         {
-            if (loop)
-                mc_search_regex__process_append_str (ret, prev_str,
-                                                     replace_str->str - prev_str + loop,
+            if (loop != 0)
+                mc_search_regex__process_append_str (ret, replace_str->str + prev, loop - prev,
                                                      &replace_flags);
-            prev_str = replace_str->str + loop + len;
-            loop += len - 1;
+            prev = loop + len;
+            loop = prev - 1;    /* prepare to loop++ */
             continue;
         }
 
         /* escape sequence */
         if (lc_index == REPLACE_PREPARE_T_ESCAPE_SEQ)
         {
-            mc_search_regex__process_append_str (ret, prev_str,
-                                                 replace_str->str + loop - prev_str,
+            mc_search_regex__process_append_str (ret, replace_str->str + prev, loop - prev,
                                                  &replace_flags);
             /* call process_escape_sequence without starting '\\' */
             mc_search_regex__process_escape_sequence (ret, replace_str->str + loop + 1, len - 1,
                                                       &replace_flags, lc_mc_search->is_utf8);
-            prev_str = replace_str->str + loop + len;
-            loop += len - 1;
+            prev = loop + len;
+            loop = prev - 1;    /* prepare to loop++ */
             continue;
         }
 
@@ -994,20 +1076,19 @@ mc_search_regex_prepare_replace_str (mc_search_t * lc_mc_search, GString * repla
         }
 
         tmp_str = mc_search_regex__get_token_by_num (lc_mc_search, lc_index);
-        if (tmp_str == NULL)
-            continue;
 
-        if (loop)
-            mc_search_regex__process_append_str (ret, prev_str, replace_str->str - prev_str + loop,
+        if (loop != 0)
+            mc_search_regex__process_append_str (ret, replace_str->str + prev, loop - prev,
                                                  &replace_flags);
-        prev_str = replace_str->str + loop + len;
 
         mc_search_regex__process_append_str (ret, tmp_str, -1, &replace_flags);
         g_free (tmp_str);
-        loop += len - 1;
+
+        prev = loop + len;
+        loop = prev - 1;        /* prepare to loop++ */
     }
-    mc_search_regex__process_append_str (ret, prev_str,
-                                         replace_str->str - prev_str + replace_str->len,
+
+    mc_search_regex__process_append_str (ret, replace_str->str + prev, replace_str->len - prev,
                                          &replace_flags);
 
     return ret;
