@@ -47,8 +47,8 @@
 /* Popup menu colors */
 #define PMENU_ENTRY_COLOR         mc_skin_color__cache[24]
 #define PMENU_SELECTED_COLOR      mc_skin_color__cache[25]
-#define PMENU_HOT_COLOR           mc_skin_color__cache[26] /* unused: not implemented yet */
-#define PMENU_HOTSEL_COLOR        mc_skin_color__cache[27] /* unused: not implemented yet */
+#define PMENU_HOT_COLOR           mc_skin_color__cache[26]      /* unused: not implemented yet */
+#define PMENU_HOTSEL_COLOR        mc_skin_color__cache[27]      /* unused: not implemented yet */
 #define PMENU_TITLE_COLOR         mc_skin_color__cache[28]
 
 #define BUTTONBAR_HOTKEY_COLOR    mc_skin_color__cache[29]
@@ -75,35 +75,39 @@
 #define HELP_TITLE_COLOR          mc_skin_color__cache[43]
 
 
-#define VIEW_BOLD_COLOR           mc_skin_color__cache[44]
-#define VIEW_UNDERLINED_COLOR     mc_skin_color__cache[45]
-#define VIEW_SELECTED_COLOR       mc_skin_color__cache[46]
+#define VIEW_NORMAL_COLOR         mc_skin_color__cache[44]
+#define VIEW_BOLD_COLOR           mc_skin_color__cache[45]
+#define VIEW_UNDERLINED_COLOR     mc_skin_color__cache[46]
+#define VIEW_SELECTED_COLOR       mc_skin_color__cache[47]
 
 /*
  * editor colors - only 4 for normal, search->found, select, and whitespace
  * respectively
  * Last is defined to view color.
  */
-#define EDITOR_NORMAL_COLOR       mc_skin_color__cache[47]
-#define EDITOR_BOLD_COLOR         mc_skin_color__cache[48]
-#define EDITOR_MARKED_COLOR       mc_skin_color__cache[49]
-#define EDITOR_WHITESPACE_COLOR   mc_skin_color__cache[50]
-#define EDITOR_RIGHT_MARGIN_COLOR mc_skin_color__cache[51]
+#define EDITOR_NORMAL_COLOR       mc_skin_color__cache[48]
+#define EDITOR_BOLD_COLOR         mc_skin_color__cache[49]
+#define EDITOR_MARKED_COLOR       mc_skin_color__cache[50]
+#define EDITOR_WHITESPACE_COLOR   mc_skin_color__cache[51]
+#define EDITOR_RIGHT_MARGIN_COLOR mc_skin_color__cache[52]
+#define EDITOR_BACKGROUND         mc_skin_color__cache[53]
+#define EDITOR_FRAME              mc_skin_color__cache[54]
+#define EDITOR_FRAME_ACTIVE       mc_skin_color__cache[55]
+#define EDITOR_FRAME_DRAG         mc_skin_color__cache[56]
 /* color of left 8 char status per line */
-#define LINE_STATE_COLOR          mc_skin_color__cache[52]
-#define BOOK_MARK_COLOR           mc_skin_color__cache[53]
-#define BOOK_MARK_FOUND_COLOR     mc_skin_color__cache[54]
+#define LINE_STATE_COLOR          mc_skin_color__cache[57]
+#define BOOK_MARK_COLOR           mc_skin_color__cache[58]
+#define BOOK_MARK_FOUND_COLOR     mc_skin_color__cache[59]
 
 /* Diff colors */
-#define DFF_ADD_COLOR             mc_skin_color__cache[55]
-#define DFF_CHG_COLOR             mc_skin_color__cache[56]
-#define DFF_CHH_COLOR             mc_skin_color__cache[57]
-#define DFF_CHD_COLOR             mc_skin_color__cache[58]
-#define DFF_DEL_COLOR             mc_skin_color__cache[59]
-#define DFF_FOLDER_COLOR          mc_skin_color__cache[60]
-#define DFF_ERROR_COLOR           mc_skin_color__cache[61]
+#define DFF_ADD_COLOR             mc_skin_color__cache[60]
+#define DFF_CHG_COLOR             mc_skin_color__cache[61]
+#define DFF_CHH_COLOR             mc_skin_color__cache[62]
+#define DFF_CHD_COLOR             mc_skin_color__cache[63]
+#define DFF_DEL_COLOR             mc_skin_color__cache[64]
+#define DFF_ERROR_COLOR           mc_skin_color__cache[65]
 
-#define MC_SKIN_COLOR_CACHE_COUNT 62
+#define MC_SKIN_COLOR_CACHE_COUNT 66
 
 /*** enums ***************************************************************************************/
 
@@ -115,6 +119,7 @@ typedef struct mc_skin_struct
     gchar *description;
     mc_config_t *config;
     GHashTable *colors;
+    gboolean have_256_colors;
 } mc_skin_t;
 
 /*** global variables defined in .c file *********************************************************/
@@ -124,7 +129,7 @@ extern mc_skin_t mc_skin__default;
 
 /*** declarations of public functions ************************************************************/
 
-gboolean mc_skin_init (GError **);
+gboolean mc_skin_init (const gchar * skin_override, GError ** error);
 void mc_skin_deinit (void);
 
 int mc_skin_color_get (const gchar *, const gchar *);
@@ -133,4 +138,6 @@ void mc_skin_lines_parse_ini_file (mc_skin_t *);
 
 gchar *mc_skin_get (const gchar *, const gchar *, const gchar *);
 
+GPtrArray *mc_skin_list (void);
+
 #endif /* MC_SKIN_H */
