@@ -1,7 +1,7 @@
 /*
    Functions for escaping and unescaping strings
 
-   Copyright (C) 2009-2014
+   Copyright (C) 2009-2015
    Free Software Foundation, Inc.
 
    Written by:
@@ -81,6 +81,8 @@ strutils_escape (const char *src, gsize src_len, const char *escaped_chars,
             case '\0':
                 g_string_append (ret, "\\0");
                 continue;
+            default:
+                break;
             }
         }
 
@@ -145,6 +147,8 @@ strutils_unescape (const char *src, gsize src_len, const char *unescaped_chars,
                 case '0':
                     g_string_append_c (ret, '\0');
                     continue;
+                default:
+                    break;
                 }
             }
 
