@@ -1,7 +1,7 @@
 /*
    8bit strings utilities
 
-   Copyright (C) 2007-2014
+   Copyright (C) 2007-2015
    Free Software Foundation, Inc.
 
    Written by:
@@ -31,52 +31,59 @@
 #include "lib/global.h"
 #include "lib/strutil.h"
 
-/* functions for singlebyte encodings, all characters have width 1
- * using standard system functions
- * there are only small differences between functions in strutil8bit.c 
- * and strutilascii.c
+/* Functions for singlebyte encodings, all characters have width 1
+ * using standard system functions.
+ * There are only small differences between functions in strutil8bit.c
+ * and strutilascii.c.
  */
 
-static const char replch = '?';
+/*** global variables ****************************************************************************/
+
+/*** file scope macro definitions ****************************************************************/
 
 /*
  * Inlines to equalize 'char' signedness for single 'char' encodings.
  * Instead of writing
- *    isspace((unsigned char)c);
+ *    isspace ((unsigned char) c);
  * you can write
- *    char_isspace(c);
+ *    char_isspace (c);
  */
-
 #define DECLARE_CTYPE_WRAPPER(func_name)       \
 static inline int char_##func_name(char c)     \
 {                                              \
     return func_name((int)(unsigned char)c);   \
 }
 
+/*** file scope type declarations ****************************************************************/
+
+/*** file scope variables ************************************************************************/
+
+static const char replch = '?';
+
+/* --------------------------------------------------------------------------------------------- */
+/*** file scope functions ************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
+
 /* *INDENT-OFF* */
 DECLARE_CTYPE_WRAPPER (isalnum)
-DECLARE_CTYPE_WRAPPER (isalpha)
-DECLARE_CTYPE_WRAPPER (isascii)
-DECLARE_CTYPE_WRAPPER (isblank)
-DECLARE_CTYPE_WRAPPER (iscntrl)
 DECLARE_CTYPE_WRAPPER (isdigit)
-DECLARE_CTYPE_WRAPPER (isgraph)
-DECLARE_CTYPE_WRAPPER (islower)
 DECLARE_CTYPE_WRAPPER (isprint)
 DECLARE_CTYPE_WRAPPER (ispunct)
 DECLARE_CTYPE_WRAPPER (isspace)
-DECLARE_CTYPE_WRAPPER (isupper)
-DECLARE_CTYPE_WRAPPER (isxdigit)
 DECLARE_CTYPE_WRAPPER (toupper)
 DECLARE_CTYPE_WRAPPER (tolower)
 /* *INDENT-ON* */
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_8bit_insert_replace_char (GString * buffer)
 {
     g_string_append_c (buffer, replch);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_is_valid_string (const char *text)
 {
@@ -84,6 +91,8 @@ str_8bit_is_valid_string (const char *text)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_is_valid_char (const char *ch, size_t size)
 {
@@ -92,18 +101,24 @@ str_8bit_is_valid_char (const char *ch, size_t size)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_8bit_cnext_char (const char **text)
 {
     (*text)++;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_8bit_cprev_char (const char **text)
 {
     (*text)--;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_cnext_noncomb_char (const char **text)
 {
@@ -114,6 +129,8 @@ str_8bit_cnext_noncomb_char (const char **text)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_cprev_noncomb_char (const char **text, const char *begin)
 {
@@ -124,36 +141,48 @@ str_8bit_cprev_noncomb_char (const char **text, const char *begin)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_isspace (const char *text)
 {
     return char_isspace (text[0]);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_ispunct (const char *text)
 {
     return char_ispunct (text[0]);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_isalnum (const char *text)
 {
     return char_isalnum (text[0]);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_isdigit (const char *text)
 {
     return char_isdigit (text[0]);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_isprint (const char *text)
 {
     return char_isprint (text[0]);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static gboolean
 str_8bit_iscombiningmark (const char *text)
 {
@@ -161,6 +190,8 @@ str_8bit_iscombiningmark (const char *text)
     return FALSE;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_toupper (const char *text, char **out, size_t * remain)
 {
@@ -173,6 +204,8 @@ str_8bit_toupper (const char *text, char **out, size_t * remain)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_tolower (const char *text, char **out, size_t * remain)
 {
@@ -185,18 +218,24 @@ str_8bit_tolower (const char *text, char **out, size_t * remain)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_length (const char *text)
 {
     return strlen (text);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_length2 (const char *text, int size)
 {
     return (size >= 0) ? min (strlen (text), (gsize) size) : strlen (text);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static gchar *
 str_8bit_conv_gerror_message (GError * mcerror, const char *def_msg)
 {
@@ -228,6 +267,8 @@ str_8bit_conv_gerror_message (GError * mcerror, const char *def_msg)
     return ret;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static estr_t
 str_8bit_vfs_convert_to (GIConv coder, const char *string, int size, GString * buffer)
 {
@@ -241,6 +282,8 @@ str_8bit_vfs_convert_to (GIConv coder, const char *string, int size, GString * b
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_8bit_term_form (const char *text)
 {
@@ -261,6 +304,8 @@ str_8bit_term_form (const char *text)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_8bit_fit_to_term (const char *text, int width, align_crt_t just_mode)
 {
@@ -286,6 +331,8 @@ str_8bit_fit_to_term (const char *text, int width, align_crt_t just_mode)
         case J_RIGHT:
             ident = width - length;
             break;
+        default:
+            break;
         }
 
         if ((int) remain <= ident)
@@ -330,6 +377,8 @@ str_8bit_fit_to_term (const char *text, int width, align_crt_t just_mode)
         case J_RIGHT:
             ident = length - width;
             break;
+        default:
+            break;
         }
 
         pos += ident;
@@ -344,6 +393,8 @@ str_8bit_fit_to_term (const char *text, int width, align_crt_t just_mode)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_8bit_term_trim (const char *text, int width)
 {
@@ -362,7 +413,6 @@ str_8bit_term_trim (const char *text, int width)
 
         if (width >= (int) length)
         {
-
             for (pos = 0; pos < length && remain > 1; pos++, actual++, remain--)
                 actual[0] = char_isprint (text[pos]) ? text[pos] : '.';
         }
@@ -386,18 +436,24 @@ str_8bit_term_trim (const char *text, int width)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_term_width2 (const char *text, size_t length)
 {
     return (length != (size_t) (-1)) ? min (strlen (text), length) : strlen (text);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_term_width1 (const char *text)
 {
     return str_8bit_term_width2 (text, (size_t) (-1));
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_term_char_width (const char *text)
 {
@@ -405,6 +461,8 @@ str_8bit_term_char_width (const char *text)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_8bit_term_substring (const char *text, int start, int width)
 {
@@ -433,6 +491,8 @@ str_8bit_term_substring (const char *text, int start, int width)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_8bit_trunc (const char *text, int width)
 {
@@ -472,6 +532,8 @@ str_8bit_trunc (const char *text, int width)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_offset_to_pos (const char *text, size_t length)
 {
@@ -479,6 +541,8 @@ str_8bit_offset_to_pos (const char *text, size_t length)
     return (int) length;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_column_to_pos (const char *text, size_t pos)
 {
@@ -486,6 +550,8 @@ str_8bit_column_to_pos (const char *text, size_t pos)
     return (int) pos;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static char *
 str_8bit_create_search_needle (const char *needle, int case_sen)
 {
@@ -493,6 +559,8 @@ str_8bit_create_search_needle (const char *needle, int case_sen)
     return (char *) needle;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_8bit_release_search_needle (char *needle, int case_sen)
 {
@@ -500,6 +568,8 @@ str_8bit_release_search_needle (char *needle, int case_sen)
     (void) needle;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static char *
 str_8bit_strdown (const char *str)
 {
@@ -516,6 +586,8 @@ str_8bit_strdown (const char *str)
     return rets;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_8bit_search_first (const char *text, const char *search, int case_sen)
 {
@@ -544,6 +616,8 @@ str_8bit_search_first (const char *text, const char *search, int case_sen)
     return match;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_8bit_search_last (const char *text, const char *search, int case_sen)
 {
@@ -572,18 +646,24 @@ str_8bit_search_last (const char *text, const char *search, int case_sen)
     return match;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_compare (const char *t1, const char *t2)
 {
     return strcmp (t1, t2);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_ncompare (const char *t1, const char *t2)
 {
     return strncmp (t1, t2, min (strlen (t1), strlen (t2)));
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_casecmp (const char *s1, const char *s2)
 {
@@ -617,6 +697,8 @@ str_8bit_casecmp (const char *s1, const char *s2)
 #endif
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_ncasecmp (const char *s1, const char *s2)
 {
@@ -656,6 +738,8 @@ str_8bit_ncasecmp (const char *s1, const char *s2)
 #endif
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_prefix (const char *text, const char *prefix)
 {
@@ -667,6 +751,8 @@ str_8bit_prefix (const char *text, const char *prefix)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_caseprefix (const char *text, const char *prefix)
 {
@@ -678,18 +764,24 @@ str_8bit_caseprefix (const char *text, const char *prefix)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_8bit_fix_string (char *text)
 {
     (void) text;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static char *
 str_8bit_create_key (const char *text, int case_sen)
 {
     return (case_sen) ? (char *) text : str_8bit_strdown (text);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_8bit_key_collate (const char *t1, const char *t2, int case_sen)
 {
@@ -699,6 +791,8 @@ str_8bit_key_collate (const char *t1, const char *t2, int case_sen)
         return strcoll (t1, t2);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_8bit_release_key (char *key, int case_sen)
 {
@@ -706,6 +800,10 @@ str_8bit_release_key (char *key, int case_sen)
         g_free (key);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+/*** public functions ****************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
+
 struct str_class
 str_8bit_init (void)
 {
@@ -761,3 +859,5 @@ str_8bit_init (void)
 
     return result;
 }
+
+/* --------------------------------------------------------------------------------------------- */
