@@ -1,8 +1,8 @@
 /*
    ASCII strings utilities
 
-   Copyright (C) 2007, 2011, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 2007-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Rostislav Benes, 2007
@@ -32,17 +32,31 @@
 #include "lib/strutil.h"
 
 /* using g_ascii function from glib
- * on terminal are showed only ascii characters (lower than 0x80) 
+ * on terminal are showed only ascii characters (lower than 0x80)
  */
 
+/*** global variables ****************************************************************************/
+
+/*** file scope macro definitions ****************************************************************/
+
+/*** file scope type declarations ****************************************************************/
+
+/*** file scope variables ************************************************************************/
+
 static const char replch = '?';
 
+/* --------------------------------------------------------------------------------------------- */
+/*** file scope functions ************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_ascii_insert_replace_char (GString * buffer)
 {
     g_string_append_c (buffer, replch);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_is_valid_string (const char *text)
 {
@@ -50,6 +64,8 @@ str_ascii_is_valid_string (const char *text)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_is_valid_char (const char *ch, size_t size)
 {
@@ -58,18 +74,24 @@ str_ascii_is_valid_char (const char *ch, size_t size)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_ascii_cnext_char (const char **text)
 {
     (*text)++;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_ascii_cprev_char (const char **text)
 {
     (*text)--;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_cnext_noncomb_char (const char **text)
 {
@@ -80,6 +102,8 @@ str_ascii_cnext_noncomb_char (const char **text)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_cprev_noncomb_char (const char **text, const char *begin)
 {
@@ -90,36 +114,48 @@ str_ascii_cprev_noncomb_char (const char **text, const char *begin)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_isspace (const char *text)
 {
     return g_ascii_isspace ((gchar) text[0]);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_ispunct (const char *text)
 {
     return g_ascii_ispunct ((gchar) text[0]);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_isalnum (const char *text)
 {
     return g_ascii_isalnum ((gchar) text[0]);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_isdigit (const char *text)
 {
     return g_ascii_isdigit ((gchar) text[0]);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_isprint (const char *text)
 {
     return g_ascii_isprint ((gchar) text[0]);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static gboolean
 str_ascii_iscombiningmark (const char *text)
 {
@@ -127,6 +163,8 @@ str_ascii_iscombiningmark (const char *text)
     return FALSE;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_toupper (const char *text, char **out, size_t * remain)
 {
@@ -139,6 +177,8 @@ str_ascii_toupper (const char *text, char **out, size_t * remain)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_tolower (const char *text, char **out, size_t * remain)
 {
@@ -151,28 +191,36 @@ str_ascii_tolower (const char *text, char **out, size_t * remain)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_length (const char *text)
 {
     return strlen (text);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_length2 (const char *text, int size)
 {
     return (size >= 0) ? min (strlen (text), (gsize) size) : strlen (text);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static gchar *
-str_ascii_conv_gerror_message (GError * error, const char *def_msg)
+str_ascii_conv_gerror_message (GError * mcerror, const char *def_msg)
 {
     /* the same as str_utf8_conv_gerror_message() */
-    if (error != NULL)
-        return g_strdup (error->message);
+    if (mcerror != NULL)
+        return g_strdup (mcerror->message);
 
     return g_strdup (def_msg != NULL ? def_msg : "");
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static estr_t
 str_ascii_vfs_convert_to (GIConv coder, const char *string, int size, GString * buffer)
 {
@@ -181,6 +229,8 @@ str_ascii_vfs_convert_to (GIConv coder, const char *string, int size, GString *
     return ESTR_SUCCESS;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_ascii_term_form (const char *text)
 {
@@ -205,6 +255,8 @@ str_ascii_term_form (const char *text)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_ascii_fit_to_term (const char *text, int width, align_crt_t just_mode)
 {
@@ -230,6 +282,8 @@ str_ascii_fit_to_term (const char *text, int width, align_crt_t just_mode)
         case J_RIGHT:
             ident = width - length;
             break;
+        default:
+            break;
         }
 
         /* add space before text */
@@ -289,6 +343,8 @@ str_ascii_fit_to_term (const char *text, int width, align_crt_t just_mode)
         case J_RIGHT:
             ident = length - width;
             break;
+        default:
+            break;
         }
 
         /* copy substring text, substring start from ident and take width 
@@ -303,10 +359,14 @@ str_ascii_fit_to_term (const char *text, int width, align_crt_t just_mode)
     }
 
   finally:
+    if (actual >= result + sizeof (result))
+        actual = result + sizeof (result) - 1;
     actual[0] = '\0';
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_ascii_term_trim (const char *text, int width)
 {
@@ -356,18 +416,24 @@ str_ascii_term_trim (const char *text, int width)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_term_width2 (const char *text, size_t length)
 {
     return (length != (size_t) (-1)) ? min (strlen (text), length) : strlen (text);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_term_width1 (const char *text)
 {
     return str_ascii_term_width2 (text, (size_t) (-1));
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_term_char_width (const char *text)
 {
@@ -375,6 +441,8 @@ str_ascii_term_char_width (const char *text)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_ascii_term_substring (const char *text, int start, int width)
 {
@@ -408,6 +476,8 @@ str_ascii_term_substring (const char *text, int start, int width)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_ascii_trunc (const char *text, int width)
 {
@@ -460,6 +530,8 @@ str_ascii_trunc (const char *text, int width)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_offset_to_pos (const char *text, size_t length)
 {
@@ -467,6 +539,8 @@ str_ascii_offset_to_pos (const char *text, size_t length)
     return (int) length;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_column_to_pos (const char *text, size_t pos)
 {
@@ -474,6 +548,8 @@ str_ascii_column_to_pos (const char *text, size_t pos)
     return (int) pos;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static char *
 str_ascii_create_search_needle (const char *needle, int case_sen)
 {
@@ -481,6 +557,8 @@ str_ascii_create_search_needle (const char *needle, int case_sen)
     return (char *) needle;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_ascii_release_search_needle (char *needle, int case_sen)
 {
@@ -489,6 +567,8 @@ str_ascii_release_search_needle (char *needle, int case_sen)
 
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_ascii_search_first (const char *text, const char *search, int case_sen)
 {
@@ -517,6 +597,8 @@ str_ascii_search_first (const char *text, const char *search, int case_sen)
     return match;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_ascii_search_last (const char *text, const char *search, int case_sen)
 {
@@ -545,30 +627,40 @@ str_ascii_search_last (const char *text, const char *search, int case_sen)
     return match;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_compare (const char *t1, const char *t2)
 {
     return strcmp (t1, t2);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_ncompare (const char *t1, const char *t2)
 {
     return strncmp (t1, t2, min (strlen (t1), strlen (t2)));
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_casecmp (const char *t1, const char *t2)
 {
     return g_ascii_strcasecmp (t1, t2);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_ncasecmp (const char *t1, const char *t2)
 {
     return g_ascii_strncasecmp (t1, t2, min (strlen (t1), strlen (t2)));
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_ascii_fix_string (char *text)
 {
@@ -576,6 +668,8 @@ str_ascii_fix_string (char *text)
         text[0] = ((unsigned char) text[0] < 128) ? text[0] : '?';
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static char *
 str_ascii_create_key (const char *text, int case_sen)
 {
@@ -583,12 +677,16 @@ str_ascii_create_key (const char *text, int case_sen)
     return (char *) text;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_key_collate (const char *t1, const char *t2, int case_sen)
 {
     return (case_sen) ? strcmp (t1, t2) : g_ascii_strcasecmp (t1, t2);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_ascii_release_key (char *key, int case_sen)
 {
@@ -596,6 +694,8 @@ str_ascii_release_key (char *key, int case_sen)
     (void) case_sen;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_prefix (const char *text, const char *prefix)
 {
@@ -607,6 +707,8 @@ str_ascii_prefix (const char *text, const char *prefix)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_ascii_caseprefix (const char *text, const char *prefix)
 {
@@ -618,6 +720,10 @@ str_ascii_caseprefix (const char *text, const char *prefix)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+/*** public functions ****************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
+
 struct str_class
 str_ascii_init (void)
 {
@@ -673,3 +779,5 @@ str_ascii_init (void)
 
     return result;
 }
+
+/* --------------------------------------------------------------------------------------------- */
