@@ -1,7 +1,7 @@
 /*
    UTF-8 strings utilities
 
-   Copyright (C) 2007-2014
+   Copyright (C) 2007-2015
    Free Software Foundation, Inc.
 
    Written by:
@@ -34,8 +34,36 @@
 
 /* using function for utf-8 from glib */
 
+/*** global variables ****************************************************************************/
+
+/*** file scope macro definitions ****************************************************************/
+
+/*** file scope type declarations ****************************************************************/
+
+struct utf8_tool
+{
+    char *actual;
+    size_t remain;
+    const char *cheked;
+    int ident;
+    gboolean compose;
+};
+
+struct term_form
+{
+    char text[BUF_MEDIUM * 6];
+    size_t width;
+    gboolean compose;
+};
+
+/*** file scope variables ************************************************************************/
+
 static const char replch[] = "\xEF\xBF\xBD";
 
+/* --------------------------------------------------------------------------------------------- */
+/*** file scope functions ************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
+
 static gboolean
 str_unichar_iscombiningmark (gunichar uni)
 {
@@ -46,18 +74,24 @@ str_unichar_iscombiningmark (gunichar uni)
         || (type == G_UNICODE_ENCLOSING_MARK) || (type == G_UNICODE_NON_SPACING_MARK);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_utf8_insert_replace_char (GString * buffer)
 {
     g_string_append (buffer, replch);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_is_valid_string (const char *text)
 {
     return g_utf8_validate (text, -1, NULL);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_is_valid_char (const char *ch, size_t size)
 {
@@ -72,18 +106,24 @@ str_utf8_is_valid_char (const char *ch, size_t size)
     }
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_utf8_cnext_char (const char **text)
 {
     (*text) = g_utf8_next_char (*text);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_utf8_cprev_char (const char **text)
 {
     (*text) = g_utf8_prev_char (*text);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_utf8_cnext_char_safe (const char **text)
 {
@@ -93,6 +133,8 @@ str_utf8_cnext_char_safe (const char **text)
         (*text)++;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_utf8_cprev_char_safe (const char **text)
 {
@@ -107,6 +149,8 @@ str_utf8_cprev_char_safe (const char **text)
         (*text)--;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_utf8_fix_string (char *text)
 {
@@ -125,6 +169,8 @@ str_utf8_fix_string (char *text)
     }
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_isspace (const char *text)
 {
@@ -134,6 +180,8 @@ str_utf8_isspace (const char *text)
     return g_unichar_isspace (uni);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_ispunct (const char *text)
 {
@@ -143,6 +191,8 @@ str_utf8_ispunct (const char *text)
     return g_unichar_ispunct (uni);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_isalnum (const char *text)
 {
@@ -152,6 +202,8 @@ str_utf8_isalnum (const char *text)
     return g_unichar_isalnum (uni);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_isdigit (const char *text)
 {
@@ -161,6 +213,8 @@ str_utf8_isdigit (const char *text)
     return g_unichar_isdigit (uni);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_isprint (const char *ch)
 {
@@ -170,6 +224,8 @@ str_utf8_isprint (const char *ch)
     return g_unichar_isprint (uni);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static gboolean
 str_utf8_iscombiningmark (const char *ch)
 {
@@ -179,6 +235,8 @@ str_utf8_iscombiningmark (const char *ch)
     return str_unichar_iscombiningmark (uni);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_cnext_noncomb_char (const char **text)
 {
@@ -195,6 +253,8 @@ str_utf8_cnext_noncomb_char (const char **text)
     return count;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_cprev_noncomb_char (const char **text, const char *begin)
 {
@@ -211,6 +271,8 @@ str_utf8_cprev_noncomb_char (const char **text, const char *begin)
     return count;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_toupper (const char *text, char **out, size_t * remain)
 {
@@ -232,6 +294,8 @@ str_utf8_toupper (const char *text, char **out, size_t * remain)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_tolower (const char *text, char **out, size_t * remain)
 {
@@ -253,6 +317,8 @@ str_utf8_tolower (const char *text, char **out, size_t * remain)
     return 1;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_length (const char *text)
 {
@@ -278,6 +344,8 @@ str_utf8_length (const char *text)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_length2 (const char *text, int size)
 {
@@ -306,6 +374,8 @@ str_utf8_length2 (const char *text, int size)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_length_noncomb (const char *text)
 {
@@ -321,26 +391,34 @@ str_utf8_length_noncomb (const char *text)
     return result;
 }
 
-/*
-   static void
-   str_utf8_questmark_sustb (char **string, size_t * left, GString * buffer)
-   {
-   char *next = g_utf8_next_char (*string);
-   (*left) -= next - (*string);
-   (*string) = next;
-   g_string_append_c (buffer, '?');
-   }
- */
+/* --------------------------------------------------------------------------------------------- */
+
+#if 0
+static void
+str_utf8_questmark_sustb (char **string, size_t * left, GString * buffer)
+{
+    char *next;
+
+    next = g_utf8_next_char (*string);
+    (*left) -= next - (*string);
+    (*string) = next;
+    g_string_append_c (buffer, '?');
+}
+#endif
+
+/* --------------------------------------------------------------------------------------------- */
 
 static gchar *
-str_utf8_conv_gerror_message (GError * error, const char *def_msg)
+str_utf8_conv_gerror_message (GError * mcerror, const char *def_msg)
 {
-    if (error != NULL)
-        return g_strdup (error->message);
+    if (mcerror != NULL)
+        return g_strdup (mcerror->message);
 
     return g_strdup (def_msg != NULL ? def_msg : "");
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static estr_t
 str_utf8_vfs_convert_to (GIConv coder, const char *string, int size, GString * buffer)
 {
@@ -354,15 +432,10 @@ str_utf8_vfs_convert_to (GIConv coder, const char *string, int size, GString * b
     return result;
 }
 
-struct term_form
-{
-    char text[BUF_MEDIUM * 6];
-    size_t width;
-    gboolean compose;
-};
+/* --------------------------------------------------------------------------------------------- */
+/* utility function, that makes string valid in utf8 and all characters printable
+ * return width of string too */
 
-/* utiliti function, that make string valid in utf8 and all characters printable
- * return width of string too*/
 static const struct term_form *
 str_utf8_make_make_term_form (const char *text, size_t length)
 {
@@ -434,6 +507,8 @@ str_utf8_make_make_term_form (const char *text, size_t length)
     return &result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_utf8_term_form (const char *text)
 {
@@ -455,16 +530,9 @@ str_utf8_term_form (const char *text)
     return result;
 }
 
-struct utf8_tool
-{
-    char *actual;
-    size_t remain;
-    const char *cheked;
-    int ident;
-    gboolean compose;
-};
+/* --------------------------------------------------------------------------------------------- */
+/* utility function, that copies all characters from checked to actual */
 
-/* utiliti function, that copy all characters from cheked to actual */
 static gboolean
 utf8_tool_copy_chars_to_end (struct utf8_tool *tool)
 {
@@ -489,8 +557,10 @@ utf8_tool_copy_chars_to_end (struct utf8_tool *tool)
     return TRUE;
 }
 
-/* utiliti function, that copy characters from cheked to actual until ident is
+/* --------------------------------------------------------------------------------------------- */
+/* utility function, that copies characters from checked to actual until ident is
  * smaller than to_ident */
+
 static gboolean
 utf8_tool_copy_chars_to (struct utf8_tool *tool, int to_ident)
 {
@@ -527,7 +597,9 @@ utf8_tool_copy_chars_to (struct utf8_tool *tool, int to_ident)
     return TRUE;
 }
 
-/* utiliti function, add count spaces to actual */
+/* --------------------------------------------------------------------------------------------- */
+/* utility function, adds count spaces to actual */
+
 static int
 utf8_tool_insert_space (struct utf8_tool *tool, int count)
 {
@@ -542,7 +614,9 @@ utf8_tool_insert_space (struct utf8_tool *tool, int count)
     return 1;
 }
 
-/* utiliti function, add one characters to actual */
+/* --------------------------------------------------------------------------------------------- */
+/* utility function, adds one characters to actual */
+
 static int
 utf8_tool_insert_char (struct utf8_tool *tool, char ch)
 {
@@ -555,8 +629,10 @@ utf8_tool_insert_char (struct utf8_tool *tool, char ch)
     return 1;
 }
 
-/* utiliti function, thah skip characters from cheked until ident is greater or
+/* --------------------------------------------------------------------------------------------- */
+/* utility function, thah skips characters from checked until ident is greater or
  * equal to to_ident */
+
 static gboolean
 utf8_tool_skip_chars_to (struct utf8_tool *tool, int to_ident)
 {
@@ -584,6 +660,8 @@ utf8_tool_skip_chars_to (struct utf8_tool *tool, int to_ident)
     return TRUE;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 utf8_tool_compose (char *buffer, size_t size)
 {
@@ -594,6 +672,8 @@ utf8_tool_compose (char *buffer, size_t size)
     g_free (composed);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_utf8_fit_to_term (const char *text, int width, align_crt_t just_mode)
 {
@@ -609,7 +689,6 @@ str_utf8_fit_to_term (const char *text, int width, align_crt_t just_mode)
 
     if (pre_form->width <= (gsize) width)
     {
-        tool.ident = 0;
         switch (HIDE_FIT (just_mode))
         {
         case J_CENTER_LEFT:
@@ -619,6 +698,9 @@ str_utf8_fit_to_term (const char *text, int width, align_crt_t just_mode)
         case J_RIGHT:
             tool.ident = width - pre_form->width;
             break;
+        default:
+            tool.ident = 0;
+            break;
         }
 
         utf8_tool_insert_space (&tool, tool.ident);
@@ -638,7 +720,6 @@ str_utf8_fit_to_term (const char *text, int width, align_crt_t just_mode)
     }
     else
     {
-        tool.ident = 0;
         switch (HIDE_FIT (just_mode))
         {
         case J_CENTER:
@@ -647,6 +728,9 @@ str_utf8_fit_to_term (const char *text, int width, align_crt_t just_mode)
         case J_RIGHT:
             tool.ident = width - pre_form->width;
             break;
+        default:
+            tool.ident = 0;
+            break;
         }
 
         utf8_tool_skip_chars_to (&tool, 0);
@@ -661,6 +745,8 @@ str_utf8_fit_to_term (const char *text, int width, align_crt_t just_mode)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_utf8_term_trim (const char *text, int width)
 {
@@ -706,6 +792,8 @@ str_utf8_term_trim (const char *text, int width)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_term_width2 (const char *text, size_t length)
 {
@@ -715,12 +803,16 @@ str_utf8_term_width2 (const char *text, size_t length)
     return result->width;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_term_width1 (const char *text)
 {
     return str_utf8_term_width2 (text, (size_t) (-1));
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_term_char_width (const char *text)
 {
@@ -730,6 +822,8 @@ str_utf8_term_char_width (const char *text)
     return (str_unichar_iscombiningmark (uni)) ? 0 : ((g_unichar_iswide (uni)) ? 2 : 1);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_utf8_term_substring (const char *text, int start, int width)
 {
@@ -759,6 +853,8 @@ str_utf8_term_substring (const char *text, int start, int width)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_utf8_trunc (const char *text, int width)
 {
@@ -792,6 +888,8 @@ str_utf8_trunc (const char *text, int width)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_offset_to_pos (const char *text, size_t length)
 {
@@ -810,6 +908,8 @@ str_utf8_offset_to_pos (const char *text, size_t length)
     }
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_column_to_pos (const char *text, size_t pos)
 {
@@ -853,6 +953,8 @@ str_utf8_column_to_pos (const char *text, size_t pos)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static char *
 str_utf8_create_search_needle (const char *needle, int case_sen)
 {
@@ -871,6 +973,8 @@ str_utf8_create_search_needle (const char *needle, int case_sen)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_utf8_release_search_needle (char *needle, int case_sen)
 {
@@ -878,6 +982,8 @@ str_utf8_release_search_needle (char *needle, int case_sen)
     g_free (needle);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_utf8_search_first (const char *text, const char *search, int case_sen)
 {
@@ -920,6 +1026,8 @@ str_utf8_search_first (const char *text, const char *search, int case_sen)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static const char *
 str_utf8_search_last (const char *text, const char *search, int case_sen)
 {
@@ -961,6 +1069,8 @@ str_utf8_search_last (const char *text, const char *search, int case_sen)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static char *
 str_utf8_normalize (const char *text)
 {
@@ -1004,6 +1114,8 @@ str_utf8_normalize (const char *text)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static char *
 str_utf8_casefold_normalize (const char *text)
 {
@@ -1053,6 +1165,8 @@ str_utf8_casefold_normalize (const char *text)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_compare (const char *t1, const char *t2)
 {
@@ -1070,6 +1184,8 @@ str_utf8_compare (const char *t1, const char *t2)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_ncompare (const char *t1, const char *t2)
 {
@@ -1090,6 +1206,8 @@ str_utf8_ncompare (const char *t1, const char *t2)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_casecmp (const char *t1, const char *t2)
 {
@@ -1107,6 +1225,8 @@ str_utf8_casecmp (const char *t1, const char *t2)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_ncasecmp (const char *t1, const char *t2)
 {
@@ -1127,6 +1247,8 @@ str_utf8_ncasecmp (const char *t1, const char *t2)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_prefix (const char *text, const char *prefix)
 {
@@ -1162,6 +1284,8 @@ str_utf8_prefix (const char *text, const char *prefix)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_caseprefix (const char *text, const char *prefix)
 {
@@ -1197,6 +1321,8 @@ str_utf8_caseprefix (const char *text, const char *prefix)
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static char *
 str_utf8_create_key_gen (const char *text, int case_sen,
                          gchar * (*keygen) (const gchar * text, gssize size))
@@ -1269,12 +1395,16 @@ str_utf8_create_key_gen (const char *text, int case_sen,
     return result;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static char *
 str_utf8_create_key (const char *text, int case_sen)
 {
     return str_utf8_create_key_gen (text, case_sen, g_utf8_collate_key);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 #ifdef MC__USE_STR_UTF8_CREATE_KEY_FOR_FILENAME
 static char *
 str_utf8_create_key_for_filename (const char *text, int case_sen)
@@ -1283,6 +1413,8 @@ str_utf8_create_key_for_filename (const char *text, int case_sen)
 }
 #endif
 
+/* --------------------------------------------------------------------------------------------- */
+
 static int
 str_utf8_key_collate (const char *t1, const char *t2, int case_sen)
 {
@@ -1290,6 +1422,8 @@ str_utf8_key_collate (const char *t1, const char *t2, int case_sen)
     return strcmp (t1, t2);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
 static void
 str_utf8_release_key (char *key, int case_sen)
 {
@@ -1297,6 +1431,10 @@ str_utf8_release_key (char *key, int case_sen)
     g_free (key);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+/*** public functions ****************************************************************************/
+/* --------------------------------------------------------------------------------------------- */
+
 struct str_class
 str_utf8_init (void)
 {
@@ -1358,3 +1496,5 @@ str_utf8_init (void)
 
     return result;
 }
+
+/* --------------------------------------------------------------------------------------------- */
