@@ -1,9 +1,8 @@
 /*
    Keyboard support routines.
 
-   Copyright (C) 1994, 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005, 2006, 2007, 2009, 2010, 2011, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Miguel de Icaza, 1994, 1995
@@ -100,6 +99,7 @@ gboolean bracketed_pasting_in_progress = FALSE;
  * certain keys that may be missing from the terminal database
  */
 const key_code_name_t key_name_conv_tab[] = {
+    {ESC_CHAR, "escape", N_("Escape"), "Esc"},
     /* KEY_F(0) is not here, since we are mapping it to f10, so there is no reason
        to define f0 as well. Also, it makes Learn keys a bunch of problems :( */
     {KEY_F (1), "f1", N_("Function key 1"), "F1"},
@@ -122,26 +122,25 @@ const key_code_name_t key_name_conv_tab[] = {
     {KEY_F (18), "f18", N_("Function key 18"), "F18"},
     {KEY_F (19), "f19", N_("Function key 19"), "F19"},
     {KEY_F (20), "f20", N_("Function key 20"), "F20"},
-    {KEY_BACKSPACE, "backspace", N_("Backspace key"), "Backspace"},
-    {KEY_END, "end", N_("End key"), "End"},
-    {KEY_UP, "up", N_("Up arrow key"), "Up"},
-    {KEY_DOWN, "down", N_("Down arrow key"), "Down"},
-    {KEY_LEFT, "left", N_("Left arrow key"), "Left"},
-    {KEY_RIGHT, "right", N_("Right arrow key"), "Right"},
-    {KEY_HOME, "home", N_("Home key"), "Home"},
-    {KEY_NPAGE, "pgdn", N_("Page Down key"), "PgDn"},
-    {KEY_PPAGE, "pgup", N_("Page Up key"), "PgUp"},
-    {KEY_IC, "insert", N_("Insert key"), "Ins"},
-    {KEY_DC, "delete", N_("Delete key"), "Del"},
     {ALT ('\t'), "complete", N_("Completion/M-tab"), "Meta-Tab"},
-    {KEY_BTAB, "backtab", N_("Back Tabulation S-tab"), "Shift-Tab"},
-    {KEY_KP_ADD, "kpplus", N_("+ on keypad"), "+"},
-    {KEY_KP_SUBTRACT, "kpminus", N_("- on keypad"), "-"},
-    {(int) '/', "kpslash", N_("Slash on keypad"), "/"},
+    {KEY_BTAB, "backtab", N_("BackTab/S-tab"), "Shift-Tab"},
+    {KEY_BACKSPACE, "backspace", N_("Backspace"), "Backspace"},
+    {KEY_UP, "up", N_("Up arrow"), "Up"},
+    {KEY_DOWN, "down", N_("Down arrow"), "Down"},
+    {KEY_LEFT, "left", N_("Left arrow"), "Left"},
+    {KEY_RIGHT, "right", N_("Right arrow"), "Right"},
+    {KEY_IC, "insert", N_("Insert"), "Ins"},
+    {KEY_DC, "delete", N_("Delete"), "Del"},
+    {KEY_HOME, "home", N_("Home"), "Home"},
+    {KEY_END, "end", N_("End key"), "End"},
+    {KEY_PPAGE, "pgup", N_("Page Up"), "PgUp"},
+    {KEY_NPAGE, "pgdn", N_("Page Down"), "PgDn"},
+    {(int) '/', "kpslash", N_("/ on keypad"), "/"},
     {KEY_KP_MULTIPLY, "kpasterisk", N_("* on keypad"), "*"},
+    {KEY_KP_SUBTRACT, "kpminus", N_("- on keypad"), "-"},
+    {KEY_KP_ADD, "kpplus", N_("+ on keypad"), "+"},
 
     /* From here on, these won't be shown in Learn keys (no space) */
-    {ESC_CHAR, "escape", N_("Escape key"), "Esc"},
     {KEY_LEFT, "kpleft", N_("Left arrow keypad"), "Left"},
     {KEY_RIGHT, "kpright", N_("Right arrow keypad"), "Right"},
     {KEY_UP, "kpup", N_("Up arrow keypad"), "Up"},
@@ -161,13 +160,13 @@ const key_code_name_t key_name_conv_tab[] = {
     {KEY_C1, "c1", N_("C1 key"), "C1"},
 
     /* Alternative label */
-    {ESC_CHAR, "esc", N_("Escape key"), "Esc"},
-    {KEY_BACKSPACE, "bs", N_("Backspace key"), "Bakspace"},
-    {KEY_IC, "ins", N_("Insert key"), "Ins"},
-    {KEY_DC, "del", N_("Delete key"), "Del"},
-    {(int) '+', "plus", N_("Plus"), "+"},
-    {(int) '-', "minus", N_("Minus"), "-"},
+    {ESC_CHAR, "esc", N_("Escape"), "Esc"},
+    {KEY_BACKSPACE, "bs", N_("Backspace"), "Bakspace"},
+    {KEY_IC, "ins", N_("Insert"), "Ins"},
+    {KEY_DC, "del", N_("Delete"), "Del"},
     {(int) '*', "asterisk", N_("Asterisk"), "*"},
+    {(int) '-', "minus", N_("Minus"), "-"},
+    {(int) '+', "plus", N_("Plus"), "+"},
     {(int) '.', "dot", N_("Dot"), "."},
     {(int) '<', "lt", N_("Less than"), "<"},
     {(int) '>', "gt", N_("Great than"), ">"},
@@ -175,6 +174,7 @@ const key_code_name_t key_name_conv_tab[] = {
     {(int) ',', "comma", N_("Comma"), ","},
     {(int) '\'', "apostrophe", N_("Apostrophe"), "\'"},
     {(int) ':', "colon", N_("Colon"), ":"},
+    {(int) ';', "semicolon", N_("Semicolon"), ";"},
     {(int) '!', "exclamation", N_("Exclamation mark"), "!"},
     {(int) '?', "question", N_("Question mark"), "?"},
     {(int) '&', "ampersand", N_("Ampersand"), "&"},
@@ -1123,6 +1123,8 @@ correct_key_code (int code)
         case KEY_KP_MULTIPLY:
             c = '*';
             break;
+        default:
+            break;
         }
 
     return (mod | c);
@@ -1946,6 +1948,7 @@ int
 tty_get_event (struct Gpm_Event *event, gboolean redo_event, gboolean block)
 {
     int c;
+    int flag = 0;               /* Return value from select */
 #ifdef HAVE_LIBGPM
     static struct Gpm_Event ev; /* Mouse event */
 #endif
@@ -1978,7 +1981,6 @@ tty_get_event (struct Gpm_Event *event, gboolean redo_event, gboolean block)
     while (pending_keys == NULL)
     {
         int nfd;
-        static int flag = 0;    /* Return value from select */
         fd_set select_set;
 
         FD_ZERO (&select_set);
