@@ -2,8 +2,8 @@
    Interface to the terminal controlling library.
    Ncurses wrapper.
 
-   Copyright (C) 2005, 2006, 2007, 2009, 2011
-   The Free Software Foundation, Inc.
+   Copyright (C) 2005-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Andrew Borodin <aborodin@vmail.ru>, 2009.
@@ -49,6 +49,7 @@
 #include "tty-internal.h"       /* mc_tty_normalize_from_utf8() */
 #include "tty.h"
 #include "color-internal.h"
+#include "key.h"
 #include "mouse.h"
 #include "win.h"
 
@@ -65,7 +66,7 @@
 
 /*** file scope macro definitions ****************************************************************/
 
-#if defined(_AIX) && !defined(CTRL)
+#if !defined(CTRL)
 #define CTRL(x) ((x) & 0x1f)
 #endif
 
@@ -530,6 +531,7 @@ tty_print_anychar (int c)
     if (mc_global.utf8_display || c > 255)
     {
         int res;
+        unsigned char str[UTF8_CHAR_LEN + 1];
 
         res = g_unichar_to_utf8 (c, (char *) str);
         if (res == 0)
@@ -540,7 +542,6 @@ tty_print_anychar (int c)
         }
         else
         {
-            unsigned char str[UTF8_CHAR_LEN + 1];
             const char *s;
 
             str[res] = '\0';
