@@ -2,8 +2,8 @@
    Interface to the terminal controlling library.
    Slang wrapper.
 
-   Copyright (C) 2005, 2006, 2007, 2009, 2010, 2011
-   The Free Software Foundation, Inc.
+   Copyright (C) 2005-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Andrew Borodin <aborodin@vmail.ru>, 2009
@@ -57,8 +57,6 @@
 
 /*** global variables ****************************************************************************/
 
-extern int reset_hp_softkeys;
-
 /*** file scope macro definitions ****************************************************************/
 
 #ifndef SLTT_MAX_SCREEN_COLS
@@ -647,6 +645,12 @@ tty_print_alt_char (int c, gboolean single)
     case ACS_RTEE:
         DRAW (c, mc_tty_frm[single ? MC_TTY_FRM_RIGHTMIDDLE : MC_TTY_FRM_DRIGHTMIDDLE]);
         break;
+    case ACS_TTEE:
+        DRAW (c, mc_tty_frm[single ? MC_TTY_FRM_TOPMIDDLE : MC_TTY_FRM_DTOPMIDDLE]);
+        break;
+    case ACS_BTEE:
+        DRAW (c, mc_tty_frm[single ? MC_TTY_FRM_BOTTOMMIDDLE : MC_TTY_FRM_DBOTTOMMIDDLE]);
+        break;
     case ACS_ULCORNER:
         DRAW (c, mc_tty_frm[single ? MC_TTY_FRM_LEFTTOP : MC_TTY_FRM_DLEFTTOP]);
         break;
