@@ -67,7 +67,9 @@ extern char *tty_tgetstr (const char *name);
 
 /* {{{ Input }}} */
 
+#ifdef HAVE_SLANG
 extern int reset_hp_softkeys;
+#endif
 
 /*** declarations of public functions ************************************************************/
 
@@ -118,7 +120,9 @@ extern void tty_print_char (int c);
 extern void tty_print_alt_char (int c, gboolean single);
 extern void tty_print_anychar (int c);
 extern void tty_print_string (const char *s);
-extern void tty_printf (const char *s, ...);
+/* *INDENT-OFF* */
+extern void tty_printf (const char *s, ...) G_GNUC_PRINTF (1, 2);
+/* *INDENT-ON* */
 
 extern void tty_print_one_vline (gboolean single);
 extern void tty_print_one_hline (gboolean single);
