@@ -1,9 +1,8 @@
 /*
    Various utilities - Unix variants
 
-   Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2007, 2011, 2012, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Miguel de Icaza, 1994, 1995, 1996
@@ -45,12 +44,14 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <fcntl.h>
 #ifdef HAVE_SYS_PARAM_H
 #include <sys/param.h>
 #endif
 #include <sys/types.h>
 #include <sys/stat.h>
+#ifdef HAVE_SYS_SELECT_H
+#include <sys/select.h>
+#endif
 #include <sys/wait.h>
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
@@ -58,11 +59,12 @@
 #ifdef HAVE_GET_PROCESS_STATS
 #include <sys/procstats.h>
 #endif
-#include <unistd.h>
 #include <pwd.h>
 #include <grp.h>
 
 #include "lib/global.h"
+
+#include "lib/unixcompat.h"
 #include "lib/vfs/vfs.h"        /* VFS_ENCODING_PREFIX */
 #include "lib/strutil.h"        /* str_move() */
 #include "lib/util.h"
@@ -213,8 +215,8 @@ my_system_make_arg_array (int flags, const char *shell, char **execute_name)
 
     if ((flags & EXECUTE_AS_SHELL) != 0)
     {
-        g_ptr_array_add (args_array, g_strdup (shell));
-        g_ptr_array_add (args_array, g_strdup ("-c"));
+        g_ptr_array_add (args_array, (gpointer) shell);
+        g_ptr_array_add (args_array, (gpointer) "-c");
         *execute_name = g_strdup (shell);
     }
     else
@@ -227,65 +229,109 @@ my_system_make_arg_array (int flags, const char *shell, char **execute_name)
         else
             *execute_name = g_strndup (shell, (gsize) (shell_token - shell));
 
-        g_ptr_array_add (args_array, g_strdup (shell));
+        g_ptr_array_add (args_array, (gpointer) shell);
     }
     return args_array;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+static void
+mc_pread_stream (mc_pipe_stream_t * ps, const fd_set * fds)
+{
+    size_t buf_len;
+    ssize_t read_len;
+
+    if (!FD_ISSET (ps->fd, fds))
+    {
+        ps->len = MC_PIPE_STREAM_UNREAD;
+        return;
+    }
+
+    buf_len = (size_t) ps->len;
+
+    if (buf_len >= MC_PIPE_BUFSIZE)
+        buf_len = ps->null_term ? MC_PIPE_BUFSIZE - 1 : MC_PIPE_BUFSIZE;
+
+    do
+    {
+        read_len = read (ps->fd, ps->buf, buf_len);
+    }
+    while (read_len < 0 && errno == EINTR);
+
+    if (read_len < 0)
+    {
+        /* reading error */
+        ps->len = MC_PIPE_ERROR_READ;
+        ps->error = errno;
+    }
+    else if (read_len == 0)
+        /* EOF */
+        ps->len = MC_PIPE_STREAM_EOF;
+    else
+    {
+        /* success */
+        ps->len = read_len;
+
+        if (ps->null_term)
+            ps->buf[(size_t) ps->len] = '\0';
+    }
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /*** public functions ****************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
-char *
-get_owner (int uid)
+const char *
+get_owner (uid_t uid)
 {
     struct passwd *pwd;
     char *name;
-    static int uid_last;
+    static uid_t uid_last;
 
-    name = i_cache_match (uid, uid_cache, UID_CACHE_SIZE);
+    name = i_cache_match ((int) uid, uid_cache, UID_CACHE_SIZE);
     if (name != NULL)
         return name;
 
     pwd = getpwuid (uid);
     if (pwd != NULL)
     {
-        i_cache_add (uid, uid_cache, UID_CACHE_SIZE, pwd->pw_name, &uid_last);
+        i_cache_add ((int) uid, uid_cache, UID_CACHE_SIZE, pwd->pw_name, (int *) &uid_last);
         return pwd->pw_name;
     }
     else
     {
         static char ibuf[10];
 
-        g_snprintf (ibuf, sizeof (ibuf), "%d", uid);
+        g_snprintf (ibuf, sizeof (ibuf), "%d", (int) uid);
         return ibuf;
     }
 }
 
 /* --------------------------------------------------------------------------------------------- */
 
-char *
-get_group (int gid)
+const char *
+get_group (gid_t gid)
 {
     struct group *grp;
     char *name;
-    static int gid_last;
+    static gid_t gid_last;
 
-    name = i_cache_match (gid, gid_cache, GID_CACHE_SIZE);
+    name = i_cache_match ((int) gid, gid_cache, GID_CACHE_SIZE);
     if (name != NULL)
         return name;
 
     grp = getgrgid (gid);
     if (grp != NULL)
     {
-        i_cache_add (gid, gid_cache, GID_CACHE_SIZE, grp->gr_name, &gid_last);
+        i_cache_add ((int) gid, gid_cache, GID_CACHE_SIZE, grp->gr_name, (int *) &gid_last);
         return grp->gr_name;
     }
     else
     {
         static char gbuf[10];
 
-        g_snprintf (gbuf, sizeof (gbuf), "%d", gid);
+        g_snprintf (gbuf, sizeof (gbuf), "%d", (int) gid);
         return gbuf;
     }
 }
@@ -450,6 +496,172 @@ my_systemv_flags (int flags, const char *command, char *const argv[])
     return status;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Create pipe and run child process.
+ *
+ * @parameter command command line of child process
+ * @paremeter error contains pointer to object to handle error code and message
+ *
+ * @return newly created object of mc_pipe_t class in success, NULL otherwise
+ */
+
+mc_pipe_t *
+mc_popen (const char *command, GError ** error)
+{
+    mc_pipe_t *p;
+    char **argv;
+
+    p = g_try_new (mc_pipe_t, 1);
+    if (p == NULL)
+    {
+        mc_replace_error (error, MC_PIPE_ERROR_CREATE_PIPE, "%s",
+                          _("Cannot create pipe descriptor"));
+        goto ret_err;
+    }
+
+    if (!g_shell_parse_argv (command, NULL, &argv, error))
+    {
+        mc_replace_error (error, MC_PIPE_ERROR_PARSE_COMMAND, "%s",
+                          _("Cannot parse command for pipe"));
+        goto ret_err;
+    }
+
+    if (!g_spawn_async_with_pipes
+        (NULL, argv, NULL, G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH, NULL, NULL,
+         &p->child_pid, NULL, &p->out.fd, &p->err.fd, error))
+    {
+        mc_replace_error (error, MC_PIPE_ERROR_CREATE_PIPE_STREAM, "%s",
+                          _("Cannot create pipe streams"));
+        goto ret_err;
+    }
+
+    g_strfreev (argv);
+
+    p->out.buf[0] = '\0';
+    p->out.len = MC_PIPE_BUFSIZE;
+    p->out.null_term = FALSE;
+
+    p->err.buf[0] = '\0';
+    p->err.len = MC_PIPE_BUFSIZE;
+    p->err.null_term = FALSE;
+
+    return p;
+
+  ret_err:
+    g_free (p);
+    return NULL;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Read stdout and stderr of pipe asynchronously.
+ *
+ * @parameter p pipe descriptor
+ *
+ * The lengths of read data contain in p->out.len and p->err.len.
+ * Before read, p->xxx.len is an input:
+ *   p->xxx.len > 0:  do read stream p->xxx and store data in p->xxx.buf;
+ *   p->xxx.len <= 0: do not read stream p->xxx.
+ *
+ * After read, p->xxx.len is an output and contains the following:
+ *   p->xxx.len > 0: an actual length of read data stored in p->xxx.buf;
+ *   p->xxx.len == MC_PIPE_STREAM_EOF: EOF of stream p->xxx;
+ *   p->xxx.len == MC_PIPE_STREAM_UNREAD: stream p->xxx was not read;
+ *   p->xxx.len == MC_PIPE_ERROR_READ: reading error, and p->xxx.errno is set appropriately.
+ *
+ * @paremeter error contains pointer to object to handle error code and message
+ */
+
+void
+mc_pread (mc_pipe_t * p, GError ** error)
+{
+    gboolean read_out, read_err;
+    fd_set fds;
+    int maxfd = 0;
+    int res;
+
+    if (error != NULL)
+        *error = NULL;
+
+    read_out = p->out.fd >= 0 && p->out.len > 0;
+    read_err = p->err.fd >= 0 && p->err.len > 0;
+
+    if (!read_out && !read_err)
+    {
+        p->out.len = MC_PIPE_STREAM_UNREAD;
+        p->err.len = MC_PIPE_STREAM_UNREAD;
+        return;
+    }
+
+    FD_ZERO (&fds);
+    if (read_out)
+    {
+        FD_SET (p->out.fd, &fds);
+        maxfd = p->out.fd;
+    }
+
+    if (read_err)
+    {
+        FD_SET (p->err.fd, &fds);
+        maxfd = max (maxfd, p->err.fd);
+    }
+
+    /* no timeout */
+    res = select (maxfd + 1, &fds, NULL, NULL, NULL);
+    if (res < 0 && errno != EINTR)
+    {
+        mc_propagate_error (error, MC_PIPE_ERROR_READ,
+                            _
+                            ("Unexpected error in select() reading data from a child process:\n%s"),
+                            unix_error_string (errno));
+        return;
+    }
+
+    if (read_out)
+        mc_pread_stream (&p->out, &fds);
+    else
+        p->out.len = MC_PIPE_STREAM_UNREAD;
+
+    if (read_err)
+        mc_pread_stream (&p->err, &fds);
+    else
+        p->err.len = MC_PIPE_STREAM_UNREAD;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Close pipe and destroy pipe descriptor.
+ *
+ * @paremeter p pipe descriptor
+ * @paremeter error contains pointer to object to handle error code and message
+ */
+
+void
+mc_pclose (mc_pipe_t * p, GError ** error)
+{
+    int res;
+
+    if (p->out.fd >= 0)
+        res = close (p->out.fd);
+    if (p->err.fd >= 0)
+        res = close (p->err.fd);
+
+    do
+    {
+        int status;
+
+        res = waitpid (p->child_pid, &status, 0);
+    }
+    while (res < 0 && errno == EINTR);
+
+    if (res < 0)
+        mc_replace_error (error, MC_PIPE_ERROR_READ, _("Unexpected error in waitpid():\n%s"),
+                          unix_error_string (errno));
+
+    g_free (p);
+}
+
 /* --------------------------------------------------------------------------------------------- */
 
 /**
@@ -472,10 +684,10 @@ tilde_expand (const char *directory)
     p = directory + 1;
 
     /* d = "~" or d = "~/" */
-    if (!(*p) || (*p == PATH_SEP))
+    if (*p == '\0' || IS_PATH_SEP (*p))
     {
         passwd = getpwuid (geteuid ());
-        q = (*p == PATH_SEP) ? p + 1 : "";
+        q = IS_PATH_SEP (*p) ? p + 1 : "";
     }
     else
     {
@@ -516,8 +728,8 @@ open_error_pipe (void)
     {
         message (D_NORMAL, _("Warning"), _("Pipe failed"));
     }
-    old_error = dup (2);
-    if (old_error < 0 || close (2) || dup (error_pipe[1]) != 2)
+    old_error = dup (STDERR_FILENO);
+    if (old_error < 0 || close (STDERR_FILENO) != 0 || dup (error_pipe[1]) != STDERR_FILENO)
     {
         message (D_NORMAL, _("Warning"), _("Dup failed"));
 
@@ -577,7 +789,7 @@ close_error_pipe (int error, const char *text)
         title = _("Warning");
     if (old_error >= 0)
     {
-        if (dup2 (old_error, 2) == -1)
+        if (dup2 (old_error, STDERR_FILENO) == -1)
         {
             if (error < 0)
                 error = D_ERROR;
@@ -631,12 +843,12 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
     const size_t url_delim_len = strlen (VFS_PATH_URL_DELIMITER);
 
     /* Detect and preserve UNC paths: //server/... */
-    if ((flags & CANON_PATH_GUARDUNC) && path[0] == PATH_SEP && path[1] == PATH_SEP)
+    if ((flags & CANON_PATH_GUARDUNC) != 0 && IS_PATH_SEP (path[0]) && IS_PATH_SEP (path[1]))
     {
         p = path + 2;
-        while (p[0] && p[0] != '/')
+        while (p[0] != '\0' && !IS_PATH_SEP (p[0]))
             p++;
-        if (p[0] == '/' && p > path + 2)
+        if (IS_PATH_SEP (p[0]) && p > path + 2)
             lpath = p;
     }
 
@@ -649,10 +861,11 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
         p = lpath;
         while (*p)
         {
-            if (p[0] == PATH_SEP && p[1] == PATH_SEP && (p == lpath || *(p - 1) != ':'))
+            if (IS_PATH_SEP (p[0]) && IS_PATH_SEP (p[1]) && (p == lpath || *(p - 1) != ':'))
             {
                 s = p + 1;
-                while (*(++s) == PATH_SEP);
+                while (IS_PATH_SEP (*(++s)))
+                    ;
                 str_move (p + 1, s);
             }
             p++;
@@ -665,7 +878,7 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
         p = lpath;
         while (*p)
         {
-            if (p[0] == PATH_SEP && p[1] == '.' && p[2] == PATH_SEP)
+            if (IS_PATH_SEP (p[0]) && p[1] == '.' && IS_PATH_SEP (p[2]))
                 str_move (p, p + 2);
             else
                 p++;
@@ -678,16 +891,16 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
 
         /* Remove trailing slashes */
         p = lpath + strlen (lpath) - 1;
-        while (p > lpath && *p == PATH_SEP)
+        while (p > lpath && IS_PATH_SEP (*p))
         {
-            if (p >= lpath - (url_delim_len + 1)
+            if (p >= lpath + url_delim_len - 1
                 && strncmp (p - url_delim_len + 1, VFS_PATH_URL_DELIMITER, url_delim_len) == 0)
                 break;
             *p-- = 0;
         }
 
         /* Remove leading "./" */
-        if (lpath[0] == '.' && lpath[1] == PATH_SEP)
+        if (lpath[0] == '.' && IS_PATH_SEP (lpath[1]))
         {
             if (lpath[2] == 0)
             {
@@ -704,7 +917,7 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
         len = strlen (lpath);
         if (len < 2)
             return;
-        if (lpath[len - 1] == PATH_SEP
+        if (IS_PATH_SEP (lpath[len - 1])
             && (len < url_delim_len
                 || strncmp (lpath + len - url_delim_len, VFS_PATH_URL_DELIMITER,
                             url_delim_len) != 0))
@@ -713,7 +926,7 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
         }
         else
         {
-            if (lpath[len - 1] == '.' && lpath[len - 2] == PATH_SEP)
+            if (lpath[len - 1] == '.' && IS_PATH_SEP (lpath[len - 2]))
             {
                 if (len == 2)
                 {
@@ -738,7 +951,8 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
         p = lpath;
         while (p[0] && p[1] && p[2])
         {
-            if ((p[0] != PATH_SEP || p[1] != '.' || p[2] != '.') || (p[3] != PATH_SEP && p[3] != 0))
+            if (!IS_PATH_SEP (p[0]) || p[1] != '.' || p[2] != '.'
+                || (!IS_PATH_SEP (p[3]) && p[3] != '\0'))
             {
                 p++;
                 continue;
@@ -750,7 +964,8 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
                 && strncmp (s - url_delim_len + 2, VFS_PATH_URL_DELIMITER, url_delim_len) == 0)
             {
                 s -= (url_delim_len - 2);
-                while (s >= lpath && *s-- != PATH_SEP);
+                while (s >= lpath && !IS_PATH_SEP (*s--))
+                    ;
             }
 
             while (s >= lpath)
@@ -761,8 +976,9 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
                     char *vfs_prefix = s - url_delim_len;
                     struct vfs_class *vclass;
 
-                    while (vfs_prefix > lpath && *--vfs_prefix != PATH_SEP);
-                    if (*vfs_prefix == PATH_SEP)
+                    while (vfs_prefix > lpath && !IS_PATH_SEP (*--vfs_prefix))
+                        ;
+                    if (IS_PATH_SEP (*vfs_prefix))
                         vfs_prefix++;
                     *(s - url_delim_len) = '\0';
 
@@ -780,7 +996,7 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
                     }
                 }
 
-                if (*s == PATH_SEP)
+                if (IS_PATH_SEP (*s))
                     break;
 
                 s--;
@@ -797,7 +1013,7 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
 
             if (p[3] != 0)
             {
-                if (s == lpath && *s == PATH_SEP)
+                if (s == lpath && IS_PATH_SEP (*s))
                 {
                     /* "/../foo" -> "/foo" */
                     str_move (s + 1, p + 4);
@@ -822,17 +1038,15 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
             if (s == lpath)
             {
                 /* "token/.." -> "." */
-                if (lpath[0] != PATH_SEP)
-                {
+                if (!IS_PATH_SEP (lpath[0]))
                     lpath[0] = '.';
-                }
-                lpath[1] = 0;
+                lpath[1] = '\0';
             }
             else
             {
                 /* "foo/token/.." -> "foo" */
                 if (s == lpath + 1)
-                    s[0] = 0;
+                    s[0] = '\0';
 #ifdef HAVE_CHARSET
                 else if ((strncmp (s, VFS_ENCODING_PREFIX, enc_prefix_len) == 0)
                          && (is_supported_encoding (s + enc_prefix_len)))
@@ -843,9 +1057,9 @@ custom_canonicalize_pathname (char *path, CANON_PATH_FLAGS flags)
                     s[2] = '\0';
 
                     /* search for the previous token */
-                    /* s[-1] == PATH_SEP */
+                    /* IS_PATH_SEP (s[-1]) */
                     p = s - 1;
-                    while (p >= lpath && *p != PATH_SEP)
+                    while (p >= lpath && !IS_PATH_SEP (*p))
                         p--;
 
                     if (p >= lpath)
@@ -909,9 +1123,8 @@ mc_realpath (const char *path, char *resolved_path)
     path = copy_path;
     max_path = copy_path + PATH_MAX - 2;
     /* If it's a relative pathname use getwd for starters. */
-    if (*path != '/')
+    if (!IS_PATH_SEP (*path))
     {
-
         new_path = g_get_current_dir ();
         if (new_path == NULL)
         {
@@ -925,19 +1138,19 @@ mc_realpath (const char *path, char *resolved_path)
         }
 
         new_path += strlen (got_path);
-        if (new_path[-1] != '/')
-            *new_path++ = '/';
+        if (!IS_PATH_SEP (new_path[-1]))
+            *new_path++ = PATH_SEP;
     }
     else
     {
-        *new_path++ = '/';
+        *new_path++ = PATH_SEP;
         path++;
     }
     /* Expand each slash-separated pathname component. */
     while (*path != '\0')
     {
         /* Ignore stray "/". */
-        if (*path == '/')
+        if (IS_PATH_SEP (*path))
         {
             path++;
             continue;
@@ -945,27 +1158,28 @@ mc_realpath (const char *path, char *resolved_path)
         if (*path == '.')
         {
             /* Ignore ".". */
-            if (path[1] == '\0' || path[1] == '/')
+            if (path[1] == '\0' || IS_PATH_SEP (path[1]))
             {
                 path++;
                 continue;
             }
             if (path[1] == '.')
             {
-                if (path[2] == '\0' || path[2] == '/')
+                if (path[2] == '\0' || IS_PATH_SEP (path[2]))
                 {
                     path += 2;
                     /* Ignore ".." at root. */
                     if (new_path == got_path + 1)
                         continue;
                     /* Handle ".." by backing up. */
-                    while ((--new_path)[-1] != '/');
+                    while (!IS_PATH_SEP ((--new_path)[-1]))
+                        ;
                     continue;
                 }
             }
         }
         /* Safely copy the next pathname component. */
-        while (*path != '\0' && *path != '/')
+        while (*path != '\0' && !IS_PATH_SEP (*path))
         {
             if (path > max_path)
             {
@@ -999,12 +1213,13 @@ mc_realpath (const char *path, char *resolved_path)
         {
             /* Note: readlink doesn't add the null byte. */
             link_path[n] = '\0';
-            if (*link_path == '/')
+            if (IS_PATH_SEP (*link_path))
                 /* Start over for an absolute symlink. */
                 new_path = got_path;
             else
                 /* Otherwise back up over this component. */
-                while (*(--new_path) != '/');
+                while (!IS_PATH_SEP (*(--new_path)))
+                    ;
             /* Safe sex check. */
             if (strlen (path) + n >= PATH_MAX - 2)
             {
@@ -1017,10 +1232,10 @@ mc_realpath (const char *path, char *resolved_path)
             path = copy_path;
         }
 #endif /* S_IFLNK */
-        *new_path++ = '/';
+        *new_path++ = PATH_SEP;
     }
     /* Delete trailing slash but don't whomp a lone slash. */
-    if (new_path != got_path + 1 && new_path[-1] == '/')
+    if (new_path != got_path + 1 && IS_PATH_SEP (new_path[-1]))
         new_path--;
     /* Make sure it's null terminated. */
     *new_path = '\0';
@@ -1101,7 +1316,7 @@ mc_build_filenamev (const char *first_element, va_list args)
 
     path = g_string_new ("");
 
-    absolute = (*first_element != '\0' && *first_element == PATH_SEP);
+    absolute = IS_PATH_SEP (*first_element);
 
     do
     {
@@ -1119,10 +1334,10 @@ mc_build_filenamev (const char *first_element, va_list args)
 
             canonicalize_pathname (tmp_element);
             len = strlen (tmp_element);
-            start = (tmp_element[0] == PATH_SEP) ? tmp_element + 1 : tmp_element;
+            start = IS_PATH_SEP (tmp_element[0]) ? tmp_element + 1 : tmp_element;
 
             g_string_append (path, start);
-            if (tmp_element[len - 1] != PATH_SEP && element != NULL)
+            if (!IS_PATH_SEP (tmp_element[len - 1]) && element != NULL)
                 g_string_append_c (path, PATH_SEP);
 
             g_free (tmp_element);
