@@ -1,7 +1,7 @@
 /*
    Virtual File System: interface functions
 
-   Copyright (C) 2011-2014
+   Copyright (C) 2011-2015
    Free Software Foundation, Inc.
 
    Written by:
@@ -41,7 +41,6 @@
 #include <sys/types.h>
 #include <signal.h>
 #include <ctype.h>              /* is_digit() */
-#include <fcntl.h>
 #include <sys/stat.h>
 #include <unistd.h>
 #include <dirent.h>
@@ -60,8 +59,10 @@
 #include "gc.h"
 #include "xdirentry.h"
 
+/* TODO: move it to separate private .h */
 extern GString *vfs_str_buffer;
 extern struct vfs_class *current_vfs;
+extern struct dirent *mc_readdir_result;
 
 /*** global variables ****************************************************************************/
 
@@ -710,7 +711,7 @@ mc_chdir (const vfs_path_t * vpath)
             char *p;
 
             p = strchr (path_element->path, 0) - 1;
-            if (*p == PATH_SEP && p > path_element->path)
+            if (IS_PATH_SEP (*p) && p > path_element->path)
                 *p = '\0';
         }
 
@@ -826,10 +827,8 @@ mc_tmpdir (void)
         return tmpdir;
 
     sys_tmp = getenv ("TMPDIR");
-    if (!sys_tmp || sys_tmp[0] != '/')
-    {
+    if (sys_tmp == NULL || !IS_PATH_SEP (sys_tmp[0]))
         sys_tmp = TMPDIR_DEFAULT;
-    }
 
     pwd = getpwuid (getuid ());
 
