@@ -1,8 +1,8 @@
 /*
    Network utilities for the Midnight Commander Virtual File System.
 
-   Copyright (C) 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2005, 2007, 2011
-   The Free Software Foundation, Inc.
+   Copyright (C) 1995-2015
+   Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
 
