@@ -1,9 +1,8 @@
 /*
    Virtual File System switch code
 
-   Copyright (C) 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-   2007, 2011, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 1995-2015
+   Free Software Foundation, Inc.
 
    Written by: 1995 Miguel de Icaza
    Jakub Jelinek, 1995
@@ -33,7 +32,7 @@
  * \author Jakub Jelinek
  * \author Pavel Machek
  * \date 1995, 1998
- * \warning funtions like extfs_lstat() have right to destroy any
+ * \warning functions like extfs_lstat() have right to destroy any
  * strings you pass to them. This is acutally ok as you g_strdup what
  * you are passing to them, anyway; still, beware.
  *
@@ -44,6 +43,7 @@
 #include <config.h>
 
 #include <errno.h>
+#include <stdlib.h>
 
 #include "lib/global.h"
 #include "lib/strutil.h"
@@ -59,15 +59,18 @@
 #include "utilvfs.h"
 #include "gc.h"
 
+/* TODO: move it to the separate .h */
 extern struct dirent *mc_readdir_result;
+extern GPtrArray *vfs__classes_list;
+extern GString *vfs_str_buffer;
+extern struct vfs_class *current_vfs;
+
 /*** global variables ****************************************************************************/
 
 GPtrArray *vfs__classes_list = NULL;
-
 GString *vfs_str_buffer = NULL;
 struct vfs_class *current_vfs = NULL;
 
-
 /*** file scope macro definitions ****************************************************************/
 
 #if defined(_AIX) && !defined(NAME_MAX)
@@ -76,7 +79,7 @@ struct vfs_class *current_vfs = NULL;
 
 #define VFS_FIRST_HANDLE 100
 
-#define ISSLASH(a) (!a || (a == '/'))
+#define ISSLASH(a) (a == '\0' || IS_PATH_SEP (a))
 
 /*** file scope type declarations ****************************************************************/
 
@@ -121,7 +124,7 @@ _vfs_translate_path (const char *path, int size, GIConv defcnv, GString * buffer
 
     /* try found /#enc: */
     semi = g_strrstr_len (path, size, VFS_ENCODING_PREFIX);
-    if (semi != NULL && (semi == path || *(semi - 1) == PATH_SEP))
+    if (semi != NULL && (semi == path || IS_PATH_SEP (semi[-1])))
     {
         char encoding[16];
         const char *slash;
@@ -361,11 +364,24 @@ vfs_translate_path_n (const char *path)
 /**
  * Get current directory without any OS calls.
  *
- * @return string contain current path
+ * @return string contains current path
+ */
+
+const char *
+vfs_get_current_dir (void)
+{
+    return current_path->str;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Get current directory without any OS calls.
+ *
+ * @return newly allocated string contains current path
  */
 
 char *
-vfs_get_current_dir (void)
+vfs_get_current_dir_n (void)
 {
     return g_strdup (current_path->str);
 }
@@ -512,12 +528,13 @@ void
 vfs_print_message (const char *msg, ...)
 {
     ev_vfs_print_message_t event_data;
+    va_list ap;
 
-    va_start (event_data.ap, msg);
-    event_data.msg = msg;
+    va_start (ap, msg);
+    event_data.msg = g_strdup_vprintf (msg, ap);
+    va_end (ap);
 
     mc_event_raise (MCEVENT_GROUP_CORE, "vfs_print_message", (gpointer) & event_data);
-    va_end (event_data.ap);
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -529,24 +546,37 @@ vfs_print_message (const char *msg, ...)
 void
 vfs_setup_cwd (void)
 {
+    char *current_dir;
+    vfs_path_t *tmp_vpath;
     const vfs_path_element_t *path_element;
 
     if (vfs_get_raw_current_dir () == NULL)
     {
-        char *tmp;
+        current_dir = g_get_current_dir ();
+        vfs_set_raw_current_dir (vfs_path_from_str (current_dir));
+        g_free (current_dir);
 
-        tmp = g_get_current_dir ();
-        vfs_set_raw_current_dir (vfs_path_from_str (tmp));
-        g_free (tmp);
+        current_dir = getenv ("PWD");
+        tmp_vpath = vfs_path_from_str (current_dir);
+
+        if (tmp_vpath != NULL)
+        {
+            struct stat my_stat, my_stat2;
+
+            if (mc_global.vfs.cd_symlinks
+                && mc_stat (tmp_vpath, &my_stat) == 0
+                && mc_stat (vfs_get_raw_current_dir (), &my_stat2) == 0
+                && my_stat.st_ino == my_stat2.st_ino && my_stat.st_dev == my_stat2.st_dev)
+                vfs_set_raw_current_dir (tmp_vpath);
+            else
+                vfs_path_free (tmp_vpath);
+        }
     }
 
     path_element = vfs_path_get_by_index (vfs_get_raw_current_dir (), -1);
 
     if ((path_element->class->flags & VFSF_LOCAL) != 0)
     {
-        char *current_dir;
-        vfs_path_t *tmp_vpath;
-
         current_dir = g_get_current_dir ();
         tmp_vpath = vfs_path_from_str (current_dir);
         g_free (current_dir);
