@@ -1,9 +1,8 @@
 /*
    Widgets for the Midnight Commander
 
-   Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007, 2009, 2010, 2011
-   The Free Software Foundation, Inc.
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
    Authors:
    Radek Doulik, 1994, 1995
@@ -11,7 +10,7 @@
    Jakub Jelinek, 1995
    Andrej Borsenkow, 1996
    Norbert Warmuth, 1997
-   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010
+   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010, 2013
 
    This file is part of the Midnight Commander.
 
@@ -49,9 +48,6 @@
 
 /*** file scope macro definitions ****************************************************************/
 
-/* Currently width is hardcoded here for text mode */
-#define gauge_len 47
-
 /*** file scope type declarations ****************************************************************/
 
 /*** file scope variables ************************************************************************/
@@ -59,26 +55,30 @@
 /*** file scope functions ************************************************************************/
 
 static cb_ret_t
-gauge_callback (Widget * w, widget_msg_t msg, int parm)
+gauge_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *data)
 {
-    WGauge *g = (WGauge *) w;
-    Dlg_head *h = g->widget.owner;
+    WGauge *g = GAUGE (w);
+    WDialog *h = w->owner;
 
-    if (msg == WIDGET_INIT)
+    switch (msg)
+    {
+    case MSG_INIT:
         return MSG_HANDLED;
 
-    /* We don't want to get the focus */
-    if (msg == WIDGET_FOCUS)
+        /* We don't want to get the focus */
+    case MSG_FOCUS:
         return MSG_NOT_HANDLED;
 
-    if (msg == WIDGET_DRAW)
-    {
-        widget_move (&g->widget, 0, 0);
-        tty_setcolor (h->color[DLG_COLOR_NORMAL]);
+    case MSG_DRAW:
+        widget_move (w, 0, 0);
         if (!g->shown)
-            tty_printf ("%*s", gauge_len, "");
+        {
+            tty_setcolor (h->color[DLG_COLOR_NORMAL]);
+            tty_printf ("%*s", w->cols, "");
+        }
         else
         {
+            int gauge_len;
             int percentage, columns;
             long total = g->max;
             long done = g->current;
@@ -95,30 +95,34 @@ gauge_callback (Widget * w, widget_msg_t msg, int parm)
                 total /= 256;
                 done /= 256;
             }
+
+            gauge_len = w->cols - 7;    /* 7 positions for percentage */
+
             percentage = (200 * done / total + 1) / 2;
-            columns = (2 * (gauge_len - 7) * done / total + 1) / 2;
+            columns = (2 * gauge_len * done / total + 1) / 2;
             tty_print_char ('[');
             if (g->from_left_to_right)
             {
                 tty_setcolor (GAUGE_COLOR);
-                tty_printf ("%*s", (int) columns, "");
+                tty_printf ("%*s", columns, "");
                 tty_setcolor (h->color[DLG_COLOR_NORMAL]);
-                tty_printf ("%*s] %3d%%", (int) (gauge_len - 7 - columns), "", (int) percentage);
+                tty_printf ("%*s] %3d%%", gauge_len - columns, "", percentage);
             }
             else
             {
                 tty_setcolor (h->color[DLG_COLOR_NORMAL]);
-                tty_printf ("%*s", gauge_len - columns - 7, "");
+                tty_printf ("%*s", gauge_len - columns, "");
                 tty_setcolor (GAUGE_COLOR);
                 tty_printf ("%*s", columns, "");
                 tty_setcolor (h->color[DLG_COLOR_NORMAL]);
-                tty_printf ("] %3d%%", 100 * columns / (gauge_len - 7), percentage);
+                tty_printf ("] %3d%%", percentage);
             }
         }
         return MSG_HANDLED;
-    }
 
-    return default_proc (msg, parm);
+    default:
+        return widget_default_callback (w, sender, msg, parm, data);
+    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -126,12 +130,16 @@ gauge_callback (Widget * w, widget_msg_t msg, int parm)
 /* --------------------------------------------------------------------------------------------- */
 
 WGauge *
-gauge_new (int y, int x, gboolean shown, int max, int current)
+gauge_new (int y, int x, int cols, gboolean shown, int max, int current)
 {
     WGauge *g;
+    Widget *w;
 
     g = g_new (WGauge, 1);
-    init_widget (&g->widget, y, x, 1, gauge_len, gauge_callback, NULL);
+    w = WIDGET (g);
+    widget_init (w, y, x, 1, cols, gauge_callback, NULL);
+    widget_want_cursor (w, FALSE);
+    widget_want_hotkey (w, FALSE);
 
     g->shown = shown;
     if (max == 0)
@@ -140,9 +148,6 @@ gauge_new (int y, int x, gboolean shown, int max, int current)
     g->current = current;
     g->from_left_to_right = TRUE;
 
-    widget_want_cursor (g->widget, FALSE);
-    widget_want_hotkey (g->widget, FALSE);
-
     return g;
 }
 
@@ -158,7 +163,7 @@ gauge_set_value (WGauge * g, int max, int current)
         max = 1;                /* I do not like division by zero :) */
     g->current = current;
     g->max = max;
-    gauge_callback ((Widget *) g, WIDGET_DRAW, 0);
+    widget_redraw (WIDGET (g));
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -169,7 +174,7 @@ gauge_show (WGauge * g, gboolean shown)
     if (g->shown != shown)
     {
         g->shown = shown;
-        gauge_callback ((Widget *) g, WIDGET_DRAW, 0);
+        widget_redraw (WIDGET (g));
     }
 }
 
