@@ -1,9 +1,8 @@
 /*
    Widgets for the Midnight Commander
 
-   Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007, 2009, 2010, 2011, 2012, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
    Authors:
    Radek Doulik, 1994, 1995
@@ -41,7 +40,6 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
-#include <fcntl.h>
 
 #include "lib/global.h"
 
@@ -164,7 +162,7 @@ history_get (const char *input_name)
 /* --------------------------------------------------------------------------------------------- */
 
 /**
- * Load history form the mc_config
+ * Load history from the mc_config
  */
 GList *
 history_load (struct mc_config_t * cfg, const char *name)
@@ -368,7 +366,7 @@ history_show (GList ** history, Widget * widget, int current)
 
     /* get modified history from dialog */
     z = NULL;
-    for (hi = query_list->list; hi != NULL; hi = g_list_next (hi))
+    for (hi = listbox_get_first_link (query_list); hi != NULL; hi = g_list_next (hi))
     {
         WLEntry *entry = LENTRY (hi->data);
 
