@@ -1,9 +1,8 @@
 /*
    Widgets for the Midnight Commander
 
-   Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007, 2009, 2010, 2011, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
    Authors:
    Radek Doulik, 1994, 1995
@@ -79,6 +78,7 @@ hline_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
                 w->cols = wo->cols - 2;
             }
         }
+        return MSG_HANDLED;
 
     case MSG_FOCUS:
         /* We don't want to get the focus */
@@ -110,6 +110,10 @@ hline_callback (Widget * w, Widget * sender, widget_msg_t msg, int parm, void *d
         }
         return MSG_HANDLED;
 
+    case MSG_DESTROY:
+        g_free (l->text);
+        return MSG_HANDLED;
+
     default:
         return widget_default_callback (w, sender, msg, parm, data);
     }
