@@ -7,8 +7,8 @@
 
 /*** typedefs(not structures) and defined constants **********************************************/
 
-#define LISTBOX_APPEND_TEXT(l,h,t,d) \
-    listbox_add_item (l->list, LISTBOX_APPEND_AT_END, h, t, d)
+#define LISTBOX_APPEND_TEXT(l,h,t,d,f) \
+    listbox_add_item (l->list, LISTBOX_APPEND_AT_END, h, t, d, f)
 
 /*** enums ***************************************************************************************/
 
@@ -16,7 +16,7 @@
 
 typedef struct
 {
-    struct Dlg_head *dlg;
+    struct WDialog *dlg;
     struct WListbox *list;
 } Listbox;
 
