@@ -1,16 +1,15 @@
 /*
    Widget based utility functions.
 
-   Copyright (C) 1994, 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-   2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
    Authors:
    Miguel de Icaza, 1994, 1995, 1996
    Radek Doulik, 1994, 1995
    Jakub Jelinek, 1995
    Andrej Borsenkow, 1995
-   Andrew Borodin <aborodin@vmail.ru>, 2009, 2010, 2012, 2013
+   Andrew Borodin <aborodin@vmail.ru>, 2009-2014
 
    This file is part of the Midnight Commander.
 
@@ -57,6 +56,7 @@ static WDialog *last_query_dlg;
 
 static int sel_pos = 0;
 
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
@@ -431,6 +431,26 @@ message (int flags, const char *title, const char *text, ...)
     g_free (p);
 }
 
+/* --------------------------------------------------------------------------------------------- */
+/** Show error message box */
+
+gboolean
+mc_error_message (GError ** mcerror, int *code)
+{
+    if (mcerror == NULL || *mcerror == NULL)
+        return FALSE;
+
+    message (D_ERROR, MSG_ERROR, _("%d: %s"), (*mcerror)->code, (*mcerror)->message);
+
+    if (code != NULL)
+        *code = (*mcerror)->code;
+
+    g_error_free (*mcerror);
+    *mcerror = NULL;
+
+    return TRUE;
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /**
  * Show input dialog, background safe.
@@ -501,3 +521,189 @@ input_expand_dialog (const char *header, const char *text,
 }
 
 /* --------------------------------------------------------------------------------------------- */
+/**
+ * Create status message window object and initialize it
+ *
+ * @param title window title
+ * @param delay initial delay to raise window in seconds
+ * @param init_cb callback to initialize user-defined part of status message
+ * @param update_cb callback to update of status message
+ * @param deinit_cb callback to deinitialize user-defined part of status message
+ *
+ * @return newly allocate status message window
+ */
+
+status_msg_t *
+status_msg_create (const char *title, double delay, status_msg_cb init_cb,
+                   status_msg_update_cb update_cb, status_msg_cb deinit_cb)
+{
+    status_msg_t *sm;
+
+    sm = g_try_new (status_msg_t, 1);
+    status_msg_init (sm, title, delay, init_cb, update_cb, deinit_cb);
+
+    return sm;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Destroy status message window object
+ *
+ * @param sm status message window object
+ */
+
+void
+status_msg_destroy (status_msg_t * sm)
+{
+    status_msg_deinit (sm);
+    g_free (sm);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Initialize already created status message window object
+ *
+ * @param sm status message window object
+ * @param title window title
+ * @param delay initial delay to raise window in seconds
+ * @param init_cb callback to initialize user-defined part of status message
+ * @param update_cb callback to update of status message
+ * @param deinit_cb callback to deinitialize user-defined part of status message
+ */
+
+void
+status_msg_init (status_msg_t * sm, const char *title, double delay, status_msg_cb init_cb,
+                 status_msg_update_cb update_cb, status_msg_cb deinit_cb)
+{
+    guint64 start;
+
+    /* repaint screen to remove previous finished dialog */
+    mc_refresh ();
+
+    start = mc_timer_elapsed (mc_global.timer);
+
+    sm->dlg = dlg_create (TRUE, 0, 0, 7, min (max (40, COLS / 2), COLS), dialog_colors,
+                          NULL, NULL, NULL, title, DLG_CENTER);
+    sm->start = start;
+    sm->delay = delay * G_USEC_PER_SEC;
+    sm->block = FALSE;
+
+    sm->init = init_cb;
+    sm->update = update_cb;
+    sm->deinit = deinit_cb;
+
+    if (sm->init != NULL)
+        sm->init (sm);
+
+    if (mc_time_elapsed (&start, sm->delay))
+    {
+        /* We will manage the dialog without any help, that's why we have to call dlg_init */
+        dlg_init (sm->dlg);
+    }
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Deinitialize status message window object
+ *
+ * @param sm status message window object
+ */
+
+void
+status_msg_deinit (status_msg_t * sm)
+{
+    if (sm == NULL)
+        return;
+
+    if (sm->deinit != NULL)
+        sm->deinit (sm);
+
+    /* close and destroy dialog */
+    dlg_run_done (sm->dlg);
+    dlg_destroy (sm->dlg);
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Update status message window
+ *
+ * @param sm status message window object
+ *
+ * @return value of pressed key
+ */
+
+int
+status_msg_common_update (status_msg_t * sm)
+{
+    int c;
+    Gpm_Event event;
+
+    if (sm == NULL)
+        return B_ENTER;
+
+    /* This should not happen, but... */
+    if (sm->dlg == NULL)
+        return B_ENTER;
+
+    if (sm->dlg->state != DLG_ACTIVE)
+    {
+        /* dialog is not shown yet */
+
+        /* do not change sm->start */
+        guint64 start = sm->start;
+
+        if (mc_time_elapsed (&start, sm->delay))
+            dlg_init (sm->dlg);
+
+        return B_ENTER;
+    }
+
+    event.x = -1;               /* Don't show the GPM cursor */
+    c = tty_get_event (&event, FALSE, sm->block);
+    if (c == EV_NONE)
+        return B_ENTER;
+
+    /* Reinitialize by non-B_CANCEL value to avoid old values
+       after events other than selecting a button */
+    sm->dlg->ret_value = B_ENTER;
+    dlg_process_event (sm->dlg, c, &event);
+
+    return sm->dlg->ret_value;
+}
+
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Callback to initialize already created simple status message window object
+ *
+ * @param sm status message window object
+ */
+
+void
+simple_status_msg_init_cb (status_msg_t * sm)
+{
+    simple_status_msg_t *ssm = SIMPLE_STATUS_MSG (sm);
+    Widget *wd = WIDGET (sm->dlg);
+
+    const char *b_name = N_("&Abort");
+    int b_width;
+    int wd_width, y;
+    Widget *b;
+
+#ifdef ENABLE_NLS
+    b_name = _(b_name);
+#endif
+
+    b_width = str_term_width1 (b_name) + 4;
+    wd_width = max (wd->cols, b_width + 6);
+
+    y = 2;
+    ssm->label = label_new (y++, 3, "");
+    add_widget_autopos (sm->dlg, ssm->label, WPOS_KEEP_TOP | WPOS_CENTER_HORZ, NULL);
+    add_widget (sm->dlg, hline_new (y++, -1, -1));
+    b = WIDGET (button_new (y++, 3, B_CANCEL, NORMAL_BUTTON, b_name, NULL));
+    add_widget_autopos (sm->dlg, b, WPOS_KEEP_TOP | WPOS_CENTER_HORZ, NULL);
+
+    widget_set_size (wd, wd->y, wd->x, y + 2, wd_width);
+}
+
+/* --------------------------------------------------------------------------------------------- */
