@@ -11,7 +11,7 @@
 
    Partly rewritten by Jakub Jelinek <jakub@redhat.com>.
 
-   Copyright (C) 1994-2014
+   Copyright (C) 1994-2015
    Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
@@ -64,7 +64,9 @@
 #ifdef HAVE_SYS_IOCTL_H
 #include <sys/ioctl.h>
 #endif
+#ifdef HAVE_FCNTL_H
 #include <fcntl.h>
+#endif
 #include <termios.h>
 
 #include "lib/unixcompat.h"     /* STDERR_FILENO */
@@ -206,7 +208,6 @@ main (int argc, char **argv)
         break;
     default:
         die ();
-        break;
     }
 
     snprintf (console_name, sizeof (console_name), p, console_minor);
@@ -267,6 +268,8 @@ main (int argc, char **argv)
         case CONSOLE_CONTENTS:
             send_contents (buffer + 4, winsz.ws_col, winsz.ws_row);
             break;
+        default:
+            break;
         }
 
         if (write (1, &console_flag, 1) != 1)
