@@ -2,11 +2,11 @@
    Routines invoked by a function key
    They normally operate on the current panel.
 
-   Copyright (C) 1994-2014
+   Copyright (C) 1994-2015
    Free Software Foundation, Inc.
 
    Written by:
-   Andrew Borodin <aborodin@vmail.ru>, 2013
+   Andrew Borodin <aborodin@vmail.ru>, 2013-2015
 
    This file is part of the Midnight Commander.
 
@@ -46,7 +46,6 @@
 #endif
 #include <unistd.h>
 #include <stdlib.h>
-#include <fcntl.h>
 #include <pwd.h>
 #include <grp.h>
 #include <sys/time.h>
@@ -57,7 +56,6 @@
 #include "lib/tty/key.h"        /* ALT() macro */
 #include "lib/tty/win.h"        /* do_enter_ca_mode() */
 #include "lib/mcconfig.h"
-#include "lib/search.h"
 #include "lib/filehighlight.h"  /* MC_FHL_INI_FILE */
 #include "lib/vfs/vfs.h"
 #include "lib/fileloc.h"
@@ -100,8 +98,6 @@
 
 /*** global variables ****************************************************************************/
 
-int select_flags = SELECT_MATCH_CASE | SELECT_SHELL_PATTERNS;
-
 /*** file scope macro definitions ****************************************************************/
 
 #ifndef MAP_FILE
@@ -121,6 +117,7 @@ enum CompareMode
 static const char *machine_str = N_("Enter machine name (F1 for details):");
 #endif /* ENABLE_VFS_NET */
 
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 /**
@@ -208,78 +205,6 @@ set_panel_filter (WPanel * p)
 
 /* --------------------------------------------------------------------------------------------- */
 
-static void
-select_unselect_cmd (const char *title, const char *history_name, gboolean do_select)
-{
-    int files_only = (select_flags & SELECT_FILES_ONLY) != 0;
-    int case_sens = (select_flags & SELECT_MATCH_CASE) != 0;
-    int shell_patterns = (select_flags & SELECT_SHELL_PATTERNS) != 0;
-
-    char *reg_exp;
-    mc_search_t *search;
-    int i;
-
-    quick_widget_t quick_widgets[] = {
-        /* *INDENT-OFF* */
-        QUICK_INPUT (INPUT_LAST_TEXT, history_name, &reg_exp, NULL,
-                     FALSE, FALSE, INPUT_COMPLETE_FILENAMES),
-        QUICK_START_COLUMNS,
-            QUICK_CHECKBOX (N_("&Files only"), &files_only, NULL),
-            QUICK_CHECKBOX (N_("&Using shell patterns"), &shell_patterns, NULL),
-        QUICK_NEXT_COLUMN,
-            QUICK_CHECKBOX (N_("&Case sensitive"), &case_sens, NULL),
-        QUICK_STOP_COLUMNS,
-        QUICK_END
-        /* *INDENT-ON* */
-    };
-
-    quick_dialog_t qdlg = {
-        -1, -1, 50,
-        title, "[Select/Unselect Files]",
-        quick_widgets, NULL, NULL
-    };
-
-    if (quick_dialog (&qdlg) == B_CANCEL)
-        return;
-
-    if (reg_exp == NULL || *reg_exp == '\0')
-    {
-        g_free (reg_exp);
-        return;
-    }
-
-    search = mc_search_new (reg_exp, -1, NULL);
-    search->search_type = (shell_patterns != 0) ? MC_SEARCH_T_GLOB : MC_SEARCH_T_REGEX;
-    search->is_entire_line = TRUE;
-    search->is_case_sensitive = case_sens != 0;
-
-    for (i = 0; i < current_panel->dir.len; i++)
-    {
-        if (DIR_IS_DOTDOT (current_panel->dir.list[i].fname))
-            continue;
-        if (S_ISDIR (current_panel->dir.list[i].st.st_mode) && files_only != 0)
-            continue;
-
-        if (mc_search_run (search, current_panel->dir.list[i].fname,
-                           0, current_panel->dir.list[i].fnamelen, NULL))
-            do_file_mark (current_panel, i, do_select);
-    }
-
-    mc_search_free (search);
-    g_free (reg_exp);
-
-    /* result flags */
-    select_flags = 0;
-    if (case_sens != 0)
-        select_flags |= SELECT_MATCH_CASE;
-    if (files_only != 0)
-        select_flags |= SELECT_FILES_ONLY;
-    if (shell_patterns != 0)
-        select_flags |= SELECT_SHELL_PATTERNS;
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 static int
 compare_files (const vfs_path_t * vpath1, const vfs_path_t * vpath2, off_t size)
 {
@@ -502,7 +427,7 @@ nice_cd (const char *text, const char *xtext, const char *help,
         return;
 
     machine =
-        input_dialog_help (text, xtext, help, history_name, "", strip_password,
+        input_dialog_help (text, xtext, help, history_name, INPUT_LAST_TEXT, strip_password,
                            INPUT_COMPLETE_FILENAMES | INPUT_COMPLETE_CD | INPUT_COMPLETE_HOSTNAMES |
                            INPUT_COMPLETE_USERNAMES);
     if (machine == NULL)
@@ -518,7 +443,7 @@ nice_cd (const char *text, const char *xtext, const char *help,
 
     g_free (machine);
 
-    if (*cd_path != PATH_SEP)
+    if (!IS_PATH_SEP (*cd_path))
     {
         char *tmp = cd_path;
 
@@ -541,12 +466,15 @@ nice_cd (const char *text, const char *xtext, const char *help,
 /* --------------------------------------------------------------------------------------------- */
 
 static void
-configure_panel_listing (WPanel * p, int list_type, int use_msformat, char *user, char *status)
+configure_panel_listing (WPanel * p, int list_type, int brief_cols, int use_msformat, char *user,
+                         char *status)
 {
     p->user_mini_status = use_msformat;
     p->list_type = list_type;
 
-    if (list_type == list_user || use_msformat)
+    if (list_type == list_brief)
+        p->brief_cols = brief_cols;
+    else if (list_type == list_user || use_msformat)
     {
         g_free (p->user_format);
         p->user_format = user;
@@ -613,7 +541,7 @@ set_basic_panel_listing_to (int panel_index, int listing_mode)
 
 gboolean
 view_file_at_line (const vfs_path_t * filename_vpath, gboolean plain_view, gboolean internal,
-                   long start_line)
+                   long start_line, off_t search_start, off_t search_end)
 {
     gboolean ret = TRUE;
 
@@ -636,7 +564,7 @@ view_file_at_line (const vfs_path_t * filename_vpath, gboolean plain_view, gbool
         mcview_default_nroff_flag = 0;
         mcview_default_magic_flag = 0;
 
-        ret = mcview_viewer (NULL, filename_vpath, start_line);
+        ret = mcview_viewer (NULL, filename_vpath, start_line, search_start, search_end);
 
         if (changed_hex_mode && !mcview_altered_hex_mode)
             mcview_default_hex_mode = 1;
@@ -659,7 +587,7 @@ view_file_at_line (const vfs_path_t * filename_vpath, gboolean plain_view, gbool
         ret = (regex_command (filename_vpath, view_entry) == 0);
         if (ret)
         {
-            ret = mcview_viewer (NULL, filename_vpath, start_line);
+            ret = mcview_viewer (NULL, filename_vpath, start_line, search_start, search_end);
             dialog_switch_process_pending ();
         }
     }
@@ -695,7 +623,7 @@ view_file_at_line (const vfs_path_t * filename_vpath, gboolean plain_view, gbool
 gboolean
 view_file (const vfs_path_t * filename_vpath, gboolean plain_view, gboolean internal)
 {
-    return view_file_at_line (filename_vpath, plain_view, internal, 0);
+    return view_file_at_line (filename_vpath, plain_view, internal, 0, 0, 0);
 }
 
 
@@ -759,7 +687,7 @@ view_filtered_cmd (void)
 
     if (command != NULL)
     {
-        mcview_viewer (command, NULL, 0);
+        mcview_viewer (command, NULL, 0, 0, 0);
         g_free (command);
         dialog_switch_process_pending ();
     }
@@ -937,7 +865,7 @@ mkdir_cmd (void)
     {
         vfs_path_t *absdir;
 
-        if (dir[0] == '/' || dir[0] == '~')
+        if (IS_PATH_SEP (dir[0]) || dir[0] == '~')
             absdir = vfs_path_from_str (dir);
         else
         {
@@ -1036,38 +964,6 @@ reread_cmd (void)
 
 /* --------------------------------------------------------------------------------------------- */
 
-void
-select_invert_cmd (void)
-{
-    int i;
-
-    for (i = 0; i < current_panel->dir.len; i++)
-    {
-        file_entry_t *file = &current_panel->dir.list[i];
-
-        if (!panels_options.reverse_files_only || !S_ISDIR (file->st.st_mode))
-            do_file_mark (current_panel, i, !file->f.marked);
-    }
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
-void
-select_cmd (void)
-{
-    select_unselect_cmd (_("Select"), ":select_cmd: Select ", TRUE);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
-void
-unselect_cmd (void)
-{
-    select_unselect_cmd (_("Unselect"), ":unselect_cmd: Unselect ", FALSE);
-}
-
-/* --------------------------------------------------------------------------------------------- */
-
 void
 ext_cmd (void)
 {
@@ -1115,6 +1011,7 @@ edit_mc_menu_cmd (void)
     vfs_path_t *menufile_vpath;
     int dir = 0;
 
+    query_set_sel (1);
     dir = query_dialog (_("Menu edit"),
                         _("Which menu file do you want to edit?"),
                         D_NORMAL, geteuid ()? 2 : 3, _("&Local"), _("&User"), _("&System Wide"));
@@ -1615,21 +1512,24 @@ single_dirsize_cmd (void)
         size_t dir_count = 0;
         size_t count = 0;
         uintmax_t total = 0;
-        ComputeDirSizeUI *ui;
+        dirsize_status_msg_t dsm;
         vfs_path_t *p;
 
-        ui = compute_dir_size_create_ui (FALSE);
         p = vfs_path_from_str (entry->fname);
 
-        if (compute_dir_size (p, ui, compute_dir_size_update_ui, &dir_count, &count, &total, TRUE)
-            == FILE_CONT)
+        memset (&dsm, 0, sizeof (dsm));
+        status_msg_init (STATUS_MSG (&dsm), _("Directory scanning"), 0, dirsize_status_init_cb,
+                         dirsize_status_update_cb, dirsize_status_deinit_cb);
+
+        if (compute_dir_size (p, &dsm, &dir_count, &count, &total, TRUE) == FILE_CONT)
         {
             entry->st.st_size = (off_t) total;
             entry->f.dir_size_computed = 1;
         }
 
         vfs_path_free (p);
-        compute_dir_size_destroy_ui (ui);
+
+        status_msg_deinit (STATUS_MSG (&dsm));
     }
 
     if (panels_options.mark_moves_down)
@@ -1650,9 +1550,11 @@ dirsizes_cmd (void)
 {
     WPanel *panel = current_panel;
     int i;
-    ComputeDirSizeUI *ui;
+    dirsize_status_msg_t dsm;
 
-    ui = compute_dir_size_create_ui (FALSE);
+    memset (&dsm, 0, sizeof (dsm));
+    status_msg_init (STATUS_MSG (&dsm), _("Directory scanning"), 0, dirsize_status_init_cb,
+                     dirsize_status_update_cb, dirsize_status_deinit_cb);
 
     for (i = 0; i < panel->dir.len; i++)
         if (S_ISDIR (panel->dir.list[i].st.st_mode)
@@ -1666,10 +1568,8 @@ dirsizes_cmd (void)
             gboolean ok;
 
             p = vfs_path_from_str (panel->dir.list[i].fname);
-            ok = compute_dir_size (p, ui, compute_dir_size_update_ui, &dir_count, &count, &total,
-                                   TRUE) != FILE_CONT;
+            ok = compute_dir_size (p, &dsm, &dir_count, &count, &total, TRUE) != FILE_CONT;
             vfs_path_free (p);
-
             if (ok)
                 break;
 
@@ -1677,7 +1577,7 @@ dirsizes_cmd (void)
             panel->dir.list[i].f.dir_size_computed = 1;
         }
 
-    compute_dir_size_destroy_ui (ui);
+    status_msg_deinit (STATUS_MSG (&dsm));
 
     recalculate_panel_summary (panel);
 
@@ -1746,19 +1646,19 @@ void
 change_listing_cmd (void)
 {
     int list_type;
-    int use_msformat;
+    int use_msformat, brief_cols;
     char *user, *status;
     WPanel *p = NULL;
 
     if (SELECTED_IS_PANEL)
         p = MENU_PANEL_IDX == 0 ? left_panel : right_panel;
 
-    list_type = panel_listing_box (p, &user, &status, &use_msformat, MENU_PANEL_IDX);
+    list_type = panel_listing_box (p, MENU_PANEL_IDX, &user, &status, &use_msformat, &brief_cols);
     if (list_type != -1)
     {
         switch_to_listing (MENU_PANEL_IDX);
         p = MENU_PANEL_IDX == 0 ? left_panel : right_panel;
-        configure_panel_listing (p, list_type, use_msformat, user, status);
+        configure_panel_listing (p, list_type, brief_cols, use_msformat, user, status);
     }
 }
 
