@@ -4,9 +4,8 @@
    with all the magic of the command input line, we depend on some
    help from the program's callback.
 
-   Copyright (C) 1995, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-   2007, 2011, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 1995-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Slava Zanko <slavazanko@gmail.com>, 2013
@@ -71,6 +70,10 @@ WInput *cmdline;
 
 /*** file scope variables ************************************************************************/
 
+/* Color styles command line */
+static input_colors_t command_colors;
+
+/* --------------------------------------------------------------------------------------------- */
 /*** file scope functions ************************************************************************/
 /* --------------------------------------------------------------------------------------------- */
 
@@ -112,7 +115,6 @@ examine_cd (const char *_path)
     /* Variable expansion */
     for (p = path_tilde, r = q; *p != '\0' && r < q + MC_MAXPATHLEN;)
     {
-
         switch (state)
         {
         case copy_sym:
@@ -178,6 +180,9 @@ examine_cd (const char *_path)
                 state = copy_sym;
                 break;
             }
+
+        default:
+            break;
         }
     }
 
@@ -197,7 +202,7 @@ handle_cdpath (const char *path)
     gboolean result = FALSE;
 
     /* CDPATH handling */
-    if (*path != PATH_SEP)
+    if (!IS_PATH_SEP (*path))
     {
         char *cdpath, *p;
         char c;
@@ -413,10 +418,8 @@ do_cd_command (char *orig_cmd)
             }
             sync_tree (vfs_path_as_str (current_panel->cwd_vpath));
         }
-        else if (cmd[operand_pos] == PATH_SEP)
-        {
+        else if (IS_PATH_SEP (cmd[operand_pos]))
             sync_tree (cmd + operand_pos);
-        }
         else
         {
             vfs_path_t *new_vpath;
@@ -464,14 +467,8 @@ WInput *
 command_new (int y, int x, int cols)
 {
     WInput *cmd;
-    const input_colors_t command_colors = {
-        DEFAULT_COLOR,
-        COMMAND_MARK_COLOR,
-        DEFAULT_COLOR,
-        COMMAND_HISTORY_COLOR
-    };
 
-    cmd = input_new (y, x, (int *) command_colors, cols, "", "cmdline",
+    cmd = input_new (y, x, command_colors, cols, "", "cmdline",
                      INPUT_COMPLETE_FILENAMES | INPUT_COMPLETE_VARIABLES | INPUT_COMPLETE_USERNAMES
                      | INPUT_COMPLETE_HOSTNAMES | INPUT_COMPLETE_CD | INPUT_COMPLETE_COMMANDS |
                      INPUT_COMPLETE_SHELL_ESC);
@@ -482,6 +479,20 @@ command_new (int y, int x, int cols)
     return cmd;
 }
 
+/* --------------------------------------------------------------------------------------------- */
+/**
+ * Set colors for the command line.
+ */
+
+void
+command_set_default_colors (void)
+{
+    command_colors[WINPUTC_MAIN] = DEFAULT_COLOR;
+    command_colors[WINPUTC_MARK] = COMMAND_MARK_COLOR;
+    command_colors[WINPUTC_UNCHANGED] = DEFAULT_COLOR;
+    command_colors[WINPUTC_HISTORY] = COMMAND_HISTORY_COLOR;
+}
+
 /* --------------------------------------------------------------------------------------------- */
 /**
  * Insert quoted text in input line.  The function is meant for the
@@ -497,7 +508,7 @@ command_insert (WInput * in, const char *text, gboolean insert_extra_space)
 {
     char *quoted_text;
 
-    quoted_text = name_quote (text, 1);
+    quoted_text = name_quote (text, TRUE);
     input_insert (in, quoted_text, insert_extra_space);
     g_free (quoted_text);
 }
