@@ -1,9 +1,8 @@
 /*
    Directory routines
 
-   Copyright (C) 1994, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-   2006, 2007, 2011, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Slava Zanko <slavazanko@gmail.com>, 2013
@@ -261,7 +260,7 @@ gboolean
 dir_list_grow (dir_list * list, int delta)
 {
     int size;
-    gboolean clear = FALSE;
+    gboolean clear_flag = FALSE;
 
     if (list == NULL)
         return FALSE;
@@ -273,7 +272,7 @@ dir_list_grow (dir_list * list, int delta)
     if (size <= 0)
     {
         size = DIR_LIST_MIN_SIZE;
-        clear = TRUE;
+        clear_flag = TRUE;
     }
 
     if (size != list->size)
@@ -288,7 +287,7 @@ dir_list_grow (dir_list * list, int delta)
         list->size = size;
     }
 
-    list->len = clear ? 0 : min (list->len, size);
+    list->len = clear_flag ? 0 : min (list->len, size);
 
     return TRUE;
 }
@@ -509,7 +508,7 @@ dir_list_sort (dir_list * list, GCompareFunc sort, const dir_sort_options_t * so
     file_entry_t *fentry;
     int dot_dot_found = 0;
 
-    if (list->len < 2)
+    if (list->len < 2 || sort == (GCompareFunc) unsorted)
         return;
 
     /* If there is an ".." entry the caller must take care to
@@ -538,8 +537,7 @@ dir_list_clean (dir_list * list)
         file_entry_t *fentry;
 
         fentry = &list->list[i];
-        g_free (fentry->fname);
-        fentry->fname = NULL;
+        MC_PTR_FREE (fentry->fname);
     }
 
     list->len = 0;
@@ -631,6 +629,7 @@ dir_list_load (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
     int link_to_dir, stale_link;
     struct stat st;
     file_entry_t *fentry;
+    const char *vpath_str;
 
     /* ".." (if any) must be the first entry in the list */
     if (!dir_list_init (list))
@@ -649,14 +648,10 @@ dir_list_load (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
 
     tree_store_start_check (vpath);
 
-    {
-        const char *vpath_str;
-
-        vpath_str = vfs_path_as_str (vpath);
-        /* Do not add a ".." entry to the root directory */
-        if ((vpath_str[0] == PATH_SEP) && (vpath_str[1] == '\0'))
-            list->len--;
-    }
+    vpath_str = vfs_path_as_str (vpath);
+    /* Do not add a ".." entry to the root directory */
+    if (IS_PATH_SEP (vpath_str[0]) && vpath_str[1] == '\0')
+        dir_list_clean (list);
 
     while ((dp = mc_readdir (dirp)) != NULL)
     {
@@ -726,7 +721,7 @@ dir_list_reload (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
         dfentry = &dir_copy.list[i];
 
         dfentry->fnamelen = fentry->fnamelen;
-        dfentry->fname = fentry->fname;
+        dfentry->fname = g_strndup (fentry->fname, fentry->fnamelen);
         dfentry->f.marked = fentry->f.marked;
         dfentry->f.dir_size_computed = fentry->f.dir_size_computed;
         dfentry->f.link_to_dir = fentry->f.link_to_dir;
@@ -740,16 +735,20 @@ dir_list_reload (dir_list * list, const vfs_path_t * vpath, GCompareFunc sort,
         }
     }
 
+    /* save len for later dir_list_clean() */
+    dir_copy.len = list->len;
+
     /* Add ".." except to the root directory. The ".." entry
        (if any) must be the first in the list. */
     tmp_path = vfs_path_get_by_index (vpath, 0)->path;
-    if (vfs_path_elements_count (vpath) == 1 && tmp_path[0] == PATH_SEP && tmp_path[1] == '\0')
+    if (vfs_path_elements_count (vpath) == 1 && IS_PATH_SEP (tmp_path[0]) && tmp_path[1] == '\0')
     {
         /* root directory */
         dir_list_clean (list);
     }
     else
     {
+        dir_list_clean (list);
         if (!dir_list_init (list))
         {
             dir_list_clean (&dir_copy);
