@@ -20,18 +20,18 @@ typedef struct WTree WTree;
 extern WTree *the_tree;
 extern int xtree_mode;
 
-struct Dlg_head;
+struct WDialog;
 
 /*** declarations of public functions ************************************************************/
 
 WTree *tree_new (int y, int x, int lines, int cols, gboolean is_panel);
 
 void tree_chdir (WTree * tree, const char *dir);
-char *tree_selected_name (const WTree * tree);
+vfs_path_t *tree_selected_name (const WTree * tree);
 
 void sync_tree (const char *pathname);
 
-WTree *find_tree (struct Dlg_head *h);
+WTree *find_tree (struct WDialog *h);
 
 /*** inline functions ****************************************************************************/
 #endif /* MC__TREE_H */
