@@ -1,7 +1,7 @@
 /*
    Main program for the Midnight Commander
 
-   Copyright (C) 1994-2014
+   Copyright (C) 1994-2015
    Free Software Foundation, Inc.
 
    Written by:
@@ -47,6 +47,7 @@
 #include "lib/tty/tty.h"
 #include "lib/tty/key.h"        /* For init_key() */
 #include "lib/tty/mouse.h"      /* init_mouse() */
+#include "lib/timer.h"
 #include "lib/skin.h"
 #include "lib/filehighlight.h"
 #include "lib/fileloc.h"
@@ -238,6 +239,8 @@ main (int argc, char *argv[])
     char *config_migrate_msg;
     int exit_code = EXIT_FAILURE;
 
+    mc_global.timer = mc_timer_new ();
+
     /* We had LC_CTYPE before, LC_ALL includs LC_TYPE as well */
 #ifdef HAVE_SETLOCALE
     (void) setlocale (LC_ALL, "");
@@ -256,6 +259,7 @@ main (int argc, char *argv[])
         g_free (mc_global.tty.shell);
       startup_exit_ok:
         str_uninit_strings ();
+        mc_timer_destroy (mc_global.timer);
         return exit_code;
     }
 
@@ -374,7 +378,7 @@ main (int argc, char *argv[])
     if (mc_global.mc_run_mode == MC_RUN_FULL)
         command_set_default_colors ();
 
-    mc_error_message (&mcerror);
+    mc_error_message (&mcerror, NULL);
 
 #ifdef ENABLE_SUBSHELL
     /* Done here to ensure that the subshell doesn't  */
@@ -466,13 +470,14 @@ main (int argc, char *argv[])
     if (macros_list != NULL)
     {
         guint i;
+
         for (i = 0; i < macros_list->len; i++)
         {
             macros_t *macros;
 
             macros = &g_array_index (macros_list, struct macros_t, i);
             if (macros != NULL && macros->macro != NULL)
-                (void) g_array_free (macros->macro, FALSE);
+                (void) g_array_free (macros->macro, TRUE);
         }
         (void) g_array_free (macros_list, TRUE);
     }
@@ -497,6 +502,8 @@ main (int argc, char *argv[])
         exit_code = EXIT_FAILURE;
     }
 
+    mc_timer_destroy (mc_global.timer);
+
     (void) putchar ('\n');      /* Hack to make shell's prompt start at left of screen */
 
     return exit_code;
