@@ -1,7 +1,7 @@
 /*
    Setup loading/saving.
 
-   Copyright (C) 1994-2014
+   Copyright (C) 1994-2015
    Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
@@ -357,6 +357,7 @@ static const struct
     { "editor_check_new_line", &option_check_nl_at_eof },
     { "editor_show_right_margin", &show_right_margin },
     { "editor_group_undo", &option_group_undo },
+    { "editor_state_full_filename", &option_state_full_filename },
 #endif /* USE_INTERNAL_EDIT */
     { "editor_ask_filename_before_edit", &editor_ask_filename_before_edit },
     { "nice_rotating_dash", &nice_rotating_dash },
@@ -1377,6 +1378,8 @@ panel_load_setup (WPanel * panel, const char *section)
         }
     g_free (buffer);
 
+    panel->brief_cols = mc_config_get_int (mc_panels_config, section, "brief_cols", 2);
+
     /* User formats */
     g_free (panel->user_format);
     panel->user_format =
@@ -1409,12 +1412,14 @@ panel_save_setup (WPanel * panel, const char *section)
     mc_config_set_string (mc_panels_config, section, "sort_order", panel->sort_field->id);
 
     for (i = 0; list_types[i].key != NULL; i++)
-        if (list_types[i].list_type == panel->list_type)
+        if (list_types[i].list_type == (int) panel->list_type)
         {
             mc_config_set_string (mc_panels_config, section, "list_mode", list_types[i].key);
             break;
         }
 
+    mc_config_set_int (mc_panels_config, section, "brief_cols", panel->brief_cols);
+
     mc_config_set_string (mc_panels_config, section, "user_format", panel->user_format);
 
     for (i = 0; i < LIST_TYPES; i++)
