@@ -1,8 +1,8 @@
 /*
    Print features specific for this build
 
-   Copyright (C) 2000, 2001, 2002, 2004, 2005, 2007, 2011
-   The Free Software Foundation, Inc.
+   Copyright (C) 2000-2015
+   Free Software Foundation, Inc.
 
    This file is part of the Midnight Commander.
 
@@ -92,13 +92,13 @@ static const char *const features[] = {
     N_("With builtin Editor\n"),
 #endif
 
-#ifdef HAVE_SUBSHELL_SUPPORT
+#ifdef ENABLE_SUBSHELL
 #ifdef SUBSHELL_OPTIONAL
     N_("With optional subshell support\n"),
 #else
     N_("With subshell support as default\n"),
 #endif
-#endif /* !HAVE_SUBSHELL_SUPPORT */
+#endif /* !ENABLE_SUBSHELL */
 
 #ifdef ENABLE_BACKGROUND
     N_("With support for background operations\n"),
@@ -188,13 +188,15 @@ show_datadirs_extended (void)
     PRINTF_SECTION (_("Config directory:"), mc_global.sysconfig_dir);
     PRINTF_SECTION (_("Data directory:"), mc_global.share_data_dir);
 
+    PRINTF_SECTION (_("File extension handlers:"), EXTHELPERSDIR);
+
 #if defined ENABLE_VFS_EXTFS || defined ENABLE_VFS_FISH
     PRINTF_SECTION (_("VFS plugins and scripts:"), LIBEXECDIR);
 #ifdef ENABLE_VFS_EXTFS
-    PRINTF2 ("extfs.d:", LIBEXECDIR, MC_EXTFS_DIR "/");
+    PRINTF2 ("extfs.d:", LIBEXECDIR, MC_EXTFS_DIR PATH_SEP_STR);
 #endif
 #ifdef ENABLE_VFS_FISH
-    PRINTF2 ("fish:", LIBEXECDIR, FISH_PREFIX "/");
+    PRINTF2 ("fish:", LIBEXECDIR, FISH_PREFIX PATH_SEP_STR);
 #endif
 #endif /* ENABLE_VFS_EXTFS || defiined ENABLE_VFS_FISH */
     (void) puts ("");
@@ -203,12 +205,12 @@ show_datadirs_extended (void)
 
     PRINTF_SECTION2 (_("Config directory:"), mc_config_get_path ());
     PRINTF_SECTION2 (_("Data directory:"), mc_config_get_data_path ());
-    PRINTF ("skins:", mc_config_get_data_path (), MC_SKINS_SUBDIR "/");
+    PRINTF ("skins:", mc_config_get_data_path (), MC_SKINS_SUBDIR PATH_SEP_STR);
 #ifdef ENABLE_VFS_EXTFS
-    PRINTF ("extfs.d:", mc_config_get_data_path (), MC_EXTFS_DIR "/");
+    PRINTF ("extfs.d:", mc_config_get_data_path (), MC_EXTFS_DIR PATH_SEP_STR);
 #endif
 #ifdef ENABLE_VFS_FISH
-    PRINTF ("fish:", mc_config_get_data_path (), FISH_PREFIX "/");
+    PRINTF ("fish:", mc_config_get_data_path (), FISH_PREFIX PATH_SEP_STR);
 #endif
 #ifdef USE_INTERNAL_EDIT
     PRINTF ("mcedit macros:", mc_config_get_data_path (), MC_MACRO_FILE);
