@@ -7,7 +7,7 @@
    Parts of this program were taken from the lsdel.c and dump.c files
    written by Ted Ts'o (tytso@mit.edu) for the ext2fs package.
 
-   Copyright (C) 1995-2014
+   Copyright (C) 1995-2015
    Free Software Foundation, Inc.
 
    Written by:
@@ -48,7 +48,6 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <fcntl.h>
 
 #ifdef HAVE_EXT2FS_EXT2_FS_H
 #include <ext2fs/ext2_fs.h>
@@ -179,13 +178,13 @@ undelfs_get_path (const vfs_path_t * vpath, char **fsname, char **file)
 #if 0
     /* Strip trailing ./
      */
-    if (p - dirname > 2 && *(p - 1) == '/' && *(p - 2) == '.')
+    if (p - dirname > 2 && IS_PATH_SEP (p[-1]) && p[-2] == '.')
         *(p = p - 2) = 0;
 #endif
 
     while (p > dirname)
     {
-        if (*p == '/')
+        if (IS_PATH_SEP (*p))
         {
             char *tmp;
 
@@ -369,13 +368,13 @@ undelfs_opendir (const vfs_path_t * vpath)
         message (D_ERROR, undelfserr, _("Cannot open file %s"), ext2_fname);
         return 0;
     }
-    vfs_print_message (_("undelfs: reading inode bitmap..."));
+    vfs_print_message ("%s", _("undelfs: reading inode bitmap..."));
     if (ext2fs_read_inode_bitmap (fs))
     {
         message (D_ERROR, undelfserr, _("Cannot load inode bitmap from:\n%s"), ext2_fname);
         goto quit_opendir;
     }
-    vfs_print_message (_("undelfs: reading block bitmap..."));
+    vfs_print_message ("%s", _("undelfs: reading block bitmap..."));
     if (ext2fs_read_block_bitmap (fs))
     {
         message (D_ERROR, undelfserr, _("Cannot load block bitmap from:\n%s"), ext2_fname);
@@ -415,8 +414,6 @@ undelfs_readdir (void *vfs_info)
                     (long) delarray[readdir_ptr].ino, delarray[readdir_ptr].num_blocks);
     readdir_ptr++;
 
-    compute_namelen (&undelfs_readdir_data.dent);
-
     return &undelfs_readdir_data;
 }
 
