@@ -2,9 +2,8 @@
    Internal file viewer for the Midnight Commander
    Function for work with coordinate cache (ccache)
 
-   Copyright (C) 1994, 1995, 1996, 1998, 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2006, 2007, 2009, 2011
-   the Free Software Foundation, Inc.
+   Copyright (C) 1994-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Miguel de Icaza, 1994, 1995, 1998
@@ -142,7 +141,7 @@ mcview_coord_cache_entry_less_nroff (const coord_cache_entry_t * a, const coord_
  * smaller than ''coord'', according to the criterion ''sort_by''. */
 
 static inline size_t
-mcview_ccache_find (mcview_t * view, const coord_cache_entry_t * coord, cmp_func_t cmp_func)
+mcview_ccache_find (WView * view, const coord_cache_entry_t * coord, cmp_func_t cmp_func)
 {
     size_t base = 0;
     size_t limit = view->coord_cache->size;
@@ -209,7 +208,7 @@ coord_cache_free (coord_cache_t * cache)
 #ifdef MC_ENABLE_DEBUGGING_CODE
 
 void
-mcview_ccache_dump (mcview_t * view)
+mcview_ccache_dump (WView * view)
 {
     FILE *f;
     off_t offset, line, column, nextline_offset, filesize;
@@ -283,7 +282,7 @@ mcview_ccache_dump (mcview_t * view)
  */
 
 void
-mcview_ccache_lookup (mcview_t * view, coord_cache_entry_t * coord, enum ccache_type lookup_what)
+mcview_ccache_lookup (WView * view, coord_cache_entry_t * coord, enum ccache_type lookup_what)
 {
     size_t i;
     coord_cache_t *cache;
@@ -418,6 +417,8 @@ mcview_ccache_lookup (mcview_t * view, coord_cache_entry_t * coord, enum ccache_
         case NROFF_BACKSPACE:
             nroff_state = NROFF_CONTINUATION;
             break;
+        default:
+            break;
         }
 
         /* Cache entries must guarantee that for each i < j,
