@@ -2,7 +2,7 @@
    Internal file viewer for the Midnight Commander
    Function for paint dialogs
 
-   Copyright (C) 1994-2014
+   Copyright (C) 1994-2015
    Free Software Foundation, Inc.
 
    Written by:
@@ -74,7 +74,7 @@ mcview_search_options_t mcview_search_options = {
 /* --------------------------------------------------------------------------------------------- */
 
 gboolean
-mcview_dialog_search (mcview_t * view)
+mcview_dialog_search (WView * view)
 {
     char *exp = NULL;
     int qd_result;
@@ -165,7 +165,7 @@ mcview_dialog_search (mcview_t * view)
 /* --------------------------------------------------------------------------------------------- */
 
 gboolean
-mcview_dialog_goto (mcview_t * view, off_t * offset)
+mcview_dialog_goto (WView * view, off_t * offset)
 {
     typedef enum
     {
