@@ -2,7 +2,7 @@
    Internal file viewer for the Midnight Commander
    Common finctions (used from some other mcviewer functions)
 
-   Copyright (C) 1994-2014
+   Copyright (C) 1994-2015
    Free Software Foundation, Inc.
 
    Written by:
@@ -73,7 +73,7 @@ const off_t OFFSETTYPE_MAX = ((off_t) 1 << (OFF_T_BITWIDTH - 1)) - 1;
 /* --------------------------------------------------------------------------------------------- */
 
 void
-mcview_toggle_magic_mode (mcview_t * view)
+mcview_toggle_magic_mode (WView * view)
 {
     char *filename, *command;
     dir_list *dir;
@@ -91,7 +91,7 @@ mcview_toggle_magic_mode (mcview_t * view)
     view->dir_idx = NULL;
     mcview_done (view);
     mcview_init (view);
-    mcview_load (view, command, filename, 0);
+    mcview_load (view, command, filename, 0, 0, 0);
     view->dir = dir;
     view->dir_idx = dir_idx;
     g_free (filename);
@@ -104,11 +104,10 @@ mcview_toggle_magic_mode (mcview_t * view)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-mcview_toggle_wrap_mode (mcview_t * view)
+mcview_toggle_wrap_mode (WView * view)
 {
-    if (view->text_wrap_mode)
-        view->dpy_start = mcview_bol (view, view->dpy_start, 0);
     view->text_wrap_mode = !view->text_wrap_mode;
+    view->dpy_wrap_dirty = TRUE;
     view->dpy_bbar_dirty = TRUE;
     view->dirty++;
 }
@@ -116,10 +115,11 @@ mcview_toggle_wrap_mode (mcview_t * view)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-mcview_toggle_nroff_mode (mcview_t * view)
+mcview_toggle_nroff_mode (WView * view)
 {
     view->text_nroff_mode = !view->text_nroff_mode;
     mcview_altered_nroff_flag = 1;
+    view->dpy_wrap_dirty = TRUE;
     view->dpy_bbar_dirty = TRUE;
     view->dirty++;
 }
@@ -127,7 +127,7 @@ mcview_toggle_nroff_mode (mcview_t * view)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-mcview_toggle_hex_mode (mcview_t * view)
+mcview_toggle_hex_mode (WView * view)
 {
     view->hex_mode = !view->hex_mode;
 
@@ -144,6 +144,8 @@ mcview_toggle_hex_mode (mcview_t * view)
         widget_want_cursor (WIDGET (view), FALSE);
     }
     mcview_altered_hex_mode = 1;
+    view->dpy_paragraph_skip_lines = 0;
+    view->dpy_wrap_dirty = TRUE;
     view->dpy_bbar_dirty = TRUE;
     view->dirty++;
 }
@@ -151,7 +153,7 @@ mcview_toggle_hex_mode (mcview_t * view)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-mcview_init (mcview_t * view)
+mcview_init (WView * view)
 {
     size_t i;
 
@@ -170,6 +172,10 @@ mcview_init (mcview_t * view)
     view->coord_cache = NULL;
 
     view->dpy_start = 0;
+    view->dpy_paragraph_skip_lines = 0;
+    mcview_state_machine_init (&view->dpy_state_top, 0);
+    view->dpy_wrap_dirty = FALSE;
+    view->force_max = -1;
     view->dpy_text_column = 0;
     view->dpy_end = 0;
     view->hex_cursor = 0;
@@ -199,7 +205,7 @@ mcview_init (mcview_t * view)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-mcview_done (mcview_t * view)
+mcview_done (WView * view)
 {
     /* Save current file position */
     if (mcview_remember_file_position && view->filename_vpath != NULL)
@@ -242,8 +248,7 @@ mcview_done (mcview_t * view)
 
     mc_search_free (view->search);
     view->search = NULL;
-    g_free (view->last_search_string);
-    view->last_search_string = NULL;
+    MC_PTR_FREE (view->last_search_string);
     mcview_nroff_seq_free (&view->search_nroff_seq);
     mcview_hexedit_free_change_list (view);
 
@@ -262,7 +267,7 @@ mcview_done (mcview_t * view)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-mcview_set_codeset (mcview_t * view)
+mcview_set_codeset (WView * view)
 {
 #ifdef HAVE_CHARSET
     const char *cp_id = NULL;
@@ -282,6 +287,7 @@ mcview_set_codeset (mcview_t * view)
             view->converter = conv;
         }
         view->utf8 = (gboolean) str_isutf8 (cp_id);
+        view->dpy_wrap_dirty = TRUE;
     }
 #else
     (void) view;
@@ -292,7 +298,7 @@ mcview_set_codeset (mcview_t * view)
 
 #ifdef HAVE_CHARSET
 void
-mcview_select_encoding (mcview_t * view)
+mcview_select_encoding (WView * view)
 {
     if (do_select_codepage ())
         mcview_set_codeset (view);
@@ -302,17 +308,12 @@ mcview_select_encoding (mcview_t * view)
 /* --------------------------------------------------------------------------------------------- */
 
 void
-mcview_show_error (mcview_t * view, const char *msg)
+mcview_show_error (WView * view, const char *msg)
 {
-    mcview_close_datasource (view);
     if (mcview_is_in_panel (view))
-    {
         mcview_set_datasource_string (view, msg);
-    }
     else
-    {
         message (D_ERROR, MSG_ERROR, "%s", msg);
-    }
 }
 
 /* --------------------------------------------------------------------------------------------- */
@@ -321,7 +322,7 @@ mcview_show_error (mcview_t * view, const char *msg)
  */
 
 off_t
-mcview_bol (mcview_t * view, off_t current, off_t limit)
+mcview_bol (WView * view, off_t current, off_t limit)
 {
     int c;
     off_t filesize;
@@ -339,7 +340,7 @@ mcview_bol (mcview_t * view, off_t current, off_t limit)
         if (c == '\r')
             current--;
     }
-    while (current > 0 && current >= limit)
+    while (current > 0 && current > limit)
     {
         if (!mcview_get_byte (view, current - 1, &c))
             break;
@@ -356,7 +357,7 @@ mcview_bol (mcview_t * view, off_t current, off_t limit)
  */
 
 off_t
-mcview_eol (mcview_t * view, off_t current, off_t limit)
+mcview_eol (WView * view, off_t current, off_t limit)
 {
     int c, prev_ch = 0;
     off_t filesize;
@@ -389,7 +390,7 @@ mcview_eol (mcview_t * view, off_t current, off_t limit)
 char *
 mcview_get_title (const WDialog * h, size_t len)
 {
-    const mcview_t *view = (const mcview_t *) find_widget_type (h, mcview_callback);
+    const WView *view = (const WView *) find_widget_type (h, mcview_callback);
     const char *modified = view->hexedit_mode && (view->change_list != NULL) ? "(*) " : "    ";
     const char *file_label;
     const char *view_filename;
@@ -409,7 +410,7 @@ mcview_get_title (const WDialog * h, size_t len)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-mcview_calc_percent (mcview_t * view, off_t p)
+mcview_calc_percent (WView * view, off_t p)
 {
     const screen_dimen right = view->status_area.left + view->status_area.width;
     const screen_dimen height = view->status_area.height;
