@@ -92,7 +92,7 @@ mcview_nroff_get_char (mcview_nroff_t * nroff, int *ret_val, off_t nroff_index)
 /* --------------------------------------------------------------------------------------------- */
 
 int
-mcview__get_nroff_real_len (mcview_t * view, off_t start, off_t length)
+mcview__get_nroff_real_len (WView * view, off_t start, off_t length)
 {
     mcview_nroff_t *nroff;
     int ret = 0;
@@ -128,7 +128,7 @@ mcview__get_nroff_real_len (mcview_t * view, off_t start, off_t length)
 /* --------------------------------------------------------------------------------------------- */
 
 mcview_nroff_t *
-mcview_nroff_seq_new_num (mcview_t * view, off_t lc_index)
+mcview_nroff_seq_new_num (WView * view, off_t lc_index)
 {
     mcview_nroff_t *nroff;
 
@@ -145,7 +145,7 @@ mcview_nroff_seq_new_num (mcview_t * view, off_t lc_index)
 /* --------------------------------------------------------------------------------------------- */
 
 mcview_nroff_t *
-mcview_nroff_seq_new (mcview_t * view)
+mcview_nroff_seq_new (WView * view)
 {
     return mcview_nroff_seq_new_num (view, (off_t) 0);
 
