@@ -1,8 +1,8 @@
 /*
    libmc - check mcconfig submodule. read and write config files
 
-   Copyright (C) 2011, 2013
-   The Free Software Foundation, Inc.
+   Copyright (C) 2011-2015
+   Free Software Foundation, Inc.
 
    Written by:
    Slava Zanko <slavazanko@gmail.com>, 2011, 2013
@@ -253,11 +253,11 @@ main (void)
 
     suite_add_tcase (s, tc_core);
     sr = srunner_create (s);
-    /* srunner_set_fork_status (sr, CK_NOFORK); */
-    srunner_run_all (sr, CK_NORMAL);
+    srunner_set_log (sr, "config_string.log");
+    srunner_run_all (sr, CK_ENV);
     number_failed = srunner_ntests_failed (sr);
     srunner_free (sr);
-    return (number_failed == 0) ? 0 : 1;
+    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
