@@ -1,11 +1,11 @@
 /*
    lib/vfs - manipulations with temp files and  dirs
 
-   Copyright (C) 2012
-   The Free Software Foundation, Inc.
+   Copyright (C) 2012-2015
+   Free Software Foundation, Inc.
 
    Written by:
-   Slava Zanko <slavazanko@gmail.com>, 2012
+   Slava Zanko <slavazanko@gmail.com>, 2012, 2013
 
    This file is part of the Midnight Commander.
 
@@ -21,15 +21,11 @@
 
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
-*/
+ */
 
 #define TEST_SUITE_NAME "/lib/vfs"
 
-#include <config.h>
-
-#include <check.h>
-
-#include "lib/global.c"
+#include "tests/mctest.h"
 
 #ifndef HAVE_CHARSET
 #define HAVE_CHARSET 1
@@ -47,6 +43,9 @@
 struct vfs_s_subclass test_subclass1, test_subclass2, test_subclass3;
 struct vfs_class vfs_test_ops1, vfs_test_ops2, vfs_test_ops3;
 
+/* --------------------------------------------------------------------------------------------- */
+
+/* @Before */
 static void
 setup (void)
 {
@@ -57,6 +56,9 @@ setup (void)
     vfs_setup_work_dir ();
 }
 
+/* --------------------------------------------------------------------------------------------- */
+
+/* @After */
 static void
 teardown (void)
 {
@@ -65,55 +67,60 @@ teardown (void)
 }
 
 /* --------------------------------------------------------------------------------------------- */
+
+/* @Test */
+/* *INDENT-OFF* */
 START_TEST (test_mc_tmpdir)
+/* *INDENT-ON* */
 {
+    /* given */
     const char *tmpdir;
     const char *env_tmpdir;
 
-    tmpdir = mc_tmpdir();
-    fail_unless (
-        g_file_test (tmpdir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR),
-        "\nNo such directory: %s\n", tmpdir
-    );
-
+    /* when */
+    tmpdir = mc_tmpdir ();
     env_tmpdir = g_getenv ("MC_TMPDIR");
-    fail_unless (
-        strcmp (env_tmpdir, tmpdir) == 0,
-        "\nenv_tmpdir=%s\n    tmpdir=%s\n", env_tmpdir, tmpdir
-    );
-}
-END_TEST
 
+    /* then */
+    fail_unless (g_file_test (tmpdir, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_DIR),
+                 "\nNo such directory: %s\n", tmpdir);
+    mctest_assert_str_eq (env_tmpdir, tmpdir);
+}
+/* *INDENT-OFF* */
+END_TEST
+/* *INDENT-ON* */
+
+/* --------------------------------------------------------------------------------------------- */
+
+/* @Test */
+/* *INDENT-OFF* */
 START_TEST (test_mc_mkstemps)
+/* *INDENT-ON* */
 {
+    /* given */
     vfs_path_t *pname_vpath = NULL;
-    char *pname;
     char *begin_pname;
     int fd;
 
+    /* when */
     fd = mc_mkstemps (&pname_vpath, "mctest-", NULL);
-    if (fd == -1)
-    {
-        fail ("\nerror creating temp file!\n");
-    }
-    pname = vfs_path_to_str (pname_vpath);
-    vfs_path_free (pname_vpath);
+    begin_pname = g_build_filename (mc_tmpdir (), "mctest-", NULL);
+
+    /* then */
     close (fd);
-
-    fail_unless (
-        g_file_test (pname, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR),
-        "\nNo such file: %s\n", pname
-    );
-    unlink(pname);
-
-    begin_pname = g_build_filename (mc_tmpdir(), "mctest-", NULL);
-    fail_unless (
-        strncmp(pname, begin_pname, strlen(begin_pname)) == 0,
-        "\nstart of %s should be equal to %s\n", pname, begin_pname
-    );
-    g_free (pname);
+    mctest_assert_int_ne (fd, -1);
+    fail_unless (g_file_test
+                 (vfs_path_as_str (pname_vpath), G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR),
+                 "\nNo such file: %s\n", vfs_path_as_str (pname_vpath));
+    unlink (vfs_path_as_str (pname_vpath));
+    fail_unless (strncmp (vfs_path_as_str (pname_vpath), begin_pname, strlen (begin_pname)) == 0,
+                 "\nstart of %s should be equal to %s\n", vfs_path_as_str (pname_vpath),
+                 begin_pname);
+    vfs_path_free (pname_vpath);
 }
+/* *INDENT-OFF* */
 END_TEST
+/* *INDENT-ON* */
 
 /* --------------------------------------------------------------------------------------------- */
 
@@ -136,10 +143,10 @@ main (void)
     suite_add_tcase (s, tc_core);
     sr = srunner_create (s);
     srunner_set_log (sr, "tempdir.log");
-    srunner_run_all (sr, CK_NORMAL);
+    srunner_run_all (sr, CK_ENV);
     number_failed = srunner_ntests_failed (sr);
     srunner_free (sr);
-    return (number_failed == 0) ? 0 : 1;
+    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
