@@ -1,5 +1,5 @@
 /*
-   lib/vfs - x_basename() function testing
+   lib - x_basename() function testing
 
    Copyright (C) 2011-2015
    Free Software Foundation, Inc.
@@ -127,10 +127,10 @@ main (void)
     suite_add_tcase (s, tc_core);
     sr = srunner_create (s);
     srunner_set_log (sr, "x_basename.log");
-    srunner_run_all (sr, CK_NORMAL);
+    srunner_run_all (sr, CK_ENV);
     number_failed = srunner_ntests_failed (sr);
     srunner_free (sr);
-    return (number_failed == 0) ? 0 : 1;
+    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
