@@ -45,7 +45,6 @@ guess_message_value (void)
 }
 
 /* --------------------------------------------------------------------------------------------- */
-int rand (void);
 
 /* @ThenReturnValue */
 static gboolean rand__return_value = 0;
@@ -180,10 +179,10 @@ main (void)
     suite_add_tcase (s, tc_core);
     sr = srunner_create (s);
     srunner_set_log (sr, "cmd__get_random_hint.log");
-    srunner_run_all (sr, CK_NORMAL);
+    srunner_run_all (sr, CK_ENV);
     number_failed = srunner_ntests_failed (sr);
     srunner_free (sr);
-    return (number_failed == 0) ? 0 : 1;
+    return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
 
 /* --------------------------------------------------------------------------------------------- */
